%%********************************************************************
%% igmres: compute ideal GMRES polynomial of deg m
%%         of a matrix B.
%%
%%    min || p(B) ||_2 
%%    where p is a polynomial of degree <= m and p(0) = 1. 
%%
%%    B = a general square matrix, can be complex. 
%%----------------------------------------------------------- 
%%
%% [blk,A,C,b,X0,y0,Z0,objval,p] = igmres(B,m,feas,solve); 
%%
%% B = a square matrix, can be complex.
%% m = degree of polynomial. 
%% feas  = 1 if want feasible starting point
%%       = 0 if otherwise.
%% solve = 0 just to initialize
%%       = 1 if want to solve using sdp.m
%%       = 2 if want to solve using sdphlf.m
%%
%% p = ideal GMRES polynomial in Matlab format.
%%
%% SDPT3: version 2.1 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 7 Jul 99
%%********************************************************************

 function [blk,A,C,b,X0,y0,Z0,objval,p] = igmres(B,m,feas,solve); 
 
   if nargin <= 3; solve = 0; end;
   if nargin <= 2; feas  = 0; end; 

   N = length(B); 
   blk{1,1} = 'nondiag';  blk{1,2} = 2*N;
   if (m >= N); error('degree >= size of B'); end;

   A = cell(1,m+1); 
   [V,H,R] = orthbasis(B,m-1);
   A{1} = eye(N);
   for k = [1:m];  A{k+1} = B*V{k};  end; 
   [blk,A,C,b,X0,y0,Z0] = norm_min(A,feas,0); 
   if (solve == 1); 
      [obj,X,y,Z] = sdp(blk,A,C,b,X0,y0,Z0);
   elseif (solve == 2);
      [obj,X,y,Z] = sdphlf(blk,A,C,b,X0,y0,Z0,1,1);
   end;
   if solve;
      if ~isreal(B); y = y(1:m) + sqrt(-1)*y(m+1:2*m); end; 
      x1 = R(1:m,1:m)*y(1:m);
      objval = -obj; 
      p = [-x1(m:-1:1); 1];  
   end; 
%%====================================================================


