%%************************************************************************
%% AHOpred: Compute AHO direction (dX,dy,dZ).
%%
%% SDPT3: version 2.1 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 7 Jul 99
%%************************************************************************

 function [dX,dy,dZ,hRd,V,d,B,hX,Lsch,Usch,Psch,schur_cpu] = AHOpred(blk,A,rp,Rd,sigma,mu,schur_opt,schurfun_parms,iscmpA,normA,listA,permA,listRd,X,Z,LA,TA,PA); 

%%
%% eigenvalue decomposition of Z. 
%%
    [d,V] = blkeig(blk,Z); 
    d = ops(d,'real'); d = ops(d,'+',5*eps*ops(d,'max'));
    ONE  = ops(d,'ones'); 
    Dsch = ops(d,'rank1inv',ONE,blk); 
    Vt   = ops(V,'ctranspose'); 
%%
%% compute schur matrix =: schur
%%
    m = length(rp); hRd = []; 
    if isnumeric(schur_opt); 
       schur = zeros(m);  B = cell(size(A)); hX = cell(size(X)); 
       for p = 1:size(blk,1); 
          tstart = cputime;
          if (schur_opt(p) == 1); 
             hX{p} = Prod3(blk(p,:),Vt{p},X{p},V{p},1); 
             for k = 1:m
                if ~isempty(LA{p,k}); 
                   tmp = V{p}(PA{p,k},:); 
                   tmp = Prod2(blk(p,:),ops(LA{p,k},'ctranspose'),tmp,31); 
                   B{p,k} =Prod3(blk(p,:),ops(tmp,'ctranspose'),TA{p,k},tmp,1);
                else
                   B{p,k} = Prod3(blk(p,:),Vt{p},A{p,k},V{p},1); 
                end;
             end;
             for k = 1:m 
                tmp = Prod2(blk(p,:),hX{p},B{p,k});  tmp = ops(tmp,'sym');
                tmp = Dsch{p}.*tmp; 
                schur(:,k) = schur(:,k) + blktrace(blk(p,:),B(p,:),tmp); 
             end;
          elseif (schur_opt(p) == 2);  
             for k = 1:m
                tmp = Prod2(blk(p,:),X{p},A{p,k});   tmp = ops(tmp,'sym');
                tmp = Prod3(blk(p,:),Vt{p},tmp,V{p},1); 
                tmp = Dsch{p}.*tmp;
                tmp = Prod3(blk(p,:),V{p},tmp,Vt{p},1,listRd{p},0); 
                schur(:,k) = schur(:,k) + blktrace(blk(p,:),A(p,:),tmp); 
             end;
          end;
          schur_cpu(p) = cputime-tstart;   
       end;
    else;
       eval(['[schur,hRd] = ',schur_opt,';']);  
       tmp = spdiags(1./normA,0,m,m);
       schur = tmp*schur*tmp; 
       if ~isempty(hRd);  hRd = hRd ./normA;  end; 
    end;
    pertdiag = 1e-16*sqrt(m)*norm(diag(schur)); 
    for k=1:m; 
       schur(k,k) = schur(k,k)+pertdiag;  
    end;
    [Lsch,Usch,Psch] = lu(schur);  Psch = sparse(Psch); 
    clear schur; 
%%
%% compute r.h.s vector
%%
    hRd = zeros(m,1);  hEinvRc = zeros(m,1); 
    EinvRc = cell(size(X));  
    sigI = ops(ops(sigma*mu,'*',ONE),'spdiags',X);  
    for p = 1:size(blk,1); 
        if (schur_opt(p) == 1);
           tmp = Prod2(blk(p,:),hX(p),ops(d(p),'spdiags',X(p))); 
           tmp = ops(tmp,'sym'); 
           EinvRc(p) = ops(Dsch(p),'.*',ops(sigI(p),'-',tmp)); 
           tmp = Prod3(blk(p,:),Vt(p),Rd(p),V(p),1); 
           tmp = Prod2(blk(p,:),hX(p),tmp); tmp = ops(tmp,'sym'); 
           hRd = hRd + blktrace(blk(p,:),B(p,:),ops(Dsch(p),'.*',tmp));  
           hEinvRc = hEinvRc + blktrace(blk(p,:),B(p,:),EinvRc(p)); 
        elseif (schur_opt(p) == 2);
           tmp = Prod2(blk(p,:),X(p),Z(p));  tmp = ops(tmp,'sym'); 
           tmp = Prod3(blk(p,:),Vt(p), ops(sigI(p),'-',tmp), V(p), 1);  
           EinvRc(p) = Prod3(blk(p,:),V(p),ops(Dsch(p),'.*',tmp),Vt(p),1); 
           %%
           tmp = Prod2(blk(p,:),X(p),Rd(p)); tmp = ops(tmp,'sym');
           tmp = ops(Dsch(p),'.*',Prod3(blk(p,:),Vt(p),tmp,V(p),1)); 
           tmp = Prod3(blk(p,:),V(p),tmp,Vt(p),1,listRd(p),0);  
           %%
           hRd = hRd + blktrace(blk(p,:),A(p,:),tmp); 
           hEinvRc = hEinvRc + blktrace(blk(p,:),A(p,:),EinvRc(p)); 
        end;
    end;
    rhs = rp + hRd - hEinvRc; 
%%
%% compute (dX,dy,dZ)
%%
    dy = Usch \ (Lsch \(Psch * rhs));  
    if any(dy == inf); break; end; 
    dZ = ops(Rd,'-',Asum(blk,A,dy,iscmpA));
    dX = cell(size(X)); 
    for p = 1:size(blk,1); 
        if (schur_opt(p) == 1); 
           tmp = Prod3(blk(p,:),Vt(p),dZ(p),V(p),1); 
           tmp = Prod2(blk(p,:),hX(p),tmp); tmp = ops(tmp,'sym');  
           dX(p)= Prod3(blk(p,:),V(p),ops(EinvRc(p),'-',ops(Dsch(p),'.*',tmp)),Vt(p),1);
        elseif (schur_opt(p) == 2); 
           tmp = Prod2(blk(p,:),X(p),dZ(p)); tmp = ops(tmp,'sym'); 
           tmp = Prod3(blk(p,:),Vt(p),tmp,V(p),1); 
           dX(p)= ops(EinvRc(p),'-',Prod3(blk(p,:),V(p),ops(Dsch(p),'.*',tmp),Vt(p),1));
       end;
    end;
%%************************************************************************


