%%************************************************************************
%% AHOpredhlf: Compute AHO direction for homogeneous algorithm.
%%
%% SDPT3: version 2.1
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 7 Jul 99
%%************************************************************************

  function [dX,dy,dZ,dtau,dkap,hRd,V,d,B,hX,Lsch,Usch,Psch,schur_cpu] = AHOpredhlf(blk,A,C,b,rp,Rd,sigma,mu,eta,schur_opt,schurfun_parms,iscmpA,normA,listA,permA,listRd,tau,kap,X,Z,LA,TA,PA);

%%
%% eigenvalue decomposition of Z. 
%%
    [d,V] = blkeig(blk,Z); 
    d = ops(d,'real'); d = ops(d,'+',5*eps*ops(d,'max'));
    ONE  = ops(d,'ones'); 
    Dsch = ops(d,'rank1inv',ONE,blk); 
    Vt = ops(V,'ctranspose'); 
%%
%% compute schur matrix =: schur
%% 
    m = length(rp)-1;
    if isnumeric(schur_opt); 
       schur = zeros(m+1); 
       B = cell(size(A));  hX = cell(size(X)); hC = cell(size(X)); 
       for p = 1:size(blk,1);
          tstart =  cputime; 
          if (schur_opt(p) == 1); 
             hX{p} = Prod3(blk(p,:),Vt{p},X{p},V{p},1);  
             hC{p} = Prod3(blk(p,:),Vt{p},C{p},V{p},1); 
             for k = 1:m 
                if ~isempty(LA{p,k}); 
                   tmp = V{p}(PA{p,k},:); 
                   tmp = Prod2(blk(p,:),ops(LA{p,k},'ctranspose'),tmp,31);  
                   B{p,k} =Prod3(blk(p,:),ops(tmp,'ctranspose'),TA{p,k},tmp,1);
                else
                   B{p,k} = Prod3(blk(p,:),Vt{p},A{p,k},V{p},1); 
                end;
             end;
             for k = 1:m
                tmp = Prod2(blk(p,:),hX{p},B{p,k});  tmp = ops(tmp,'sym'); 
                tmp = Dsch{p}.*tmp;
                schur(1:m,k) = schur(1:m,k) + blktrace(blk(p,:),B(p,:),tmp); 
             end;
             tmp = Prod2(blk(p,:),hX{p},hC{p});  tmp = ops(tmp,'sym'); 
             schur(1:m,m+1) = schur(1:m,m+1) + ...
                   blktrace(blk(p,:),B(p,:),Dsch{p}.*tmp); 
             tmp = Prod2(blk(p,:),hX{p},Dsch{p}.*hC{p});  tmp = ops(tmp,'sym');
             schur(m+1,:) = schur(m+1,:) + ...
             [-blktrace(blk(p,:),B(p,:),tmp); blktrace(blk(p,:),hC(p),tmp)]'; 
          elseif (schur_opt(p) == 2); 
             for k = 1:m
                tmp = Prod2(blk(p,:),X{p},A{p,k}); tmp = ops(tmp,'sym'); 
                tmp = Prod3(blk(p,:),Vt{p},tmp,V{p},1); 
                tmp = Dsch{p}.*tmp; 
                tmp = Prod3(blk(p,:),V{p},tmp,Vt{p},1,listRd{p},0);
                schur(1:m,k) = schur(1:m,k) + blktrace(blk(p,:),A(p,:),tmp); 
             end;
             tmp = Prod2(blk(p,:),X{p},C{p});  tmp = ops(tmp,'sym'); 
             tmp = Prod3(blk(p,:),Vt{p},tmp,V{p},1); 
             tmp = Prod3(blk(p,:),V{p},Dsch{p}.*tmp,Vt{p},1,listRd{p},0);
             schur(:,m+1) = schur(:,m+1) + ...
             [-blktrace(blk(p,:),A(p,:),tmp); blktrace(blk(p,:),C(p),tmp)];  
             tmp = Prod3(blk(p,:),Vt{p},C{p},V{p},1);   
             tmp = Prod3(blk(p,:),V{p},Dsch{p}.*tmp,Vt{p},1); 
             tmp = Prod2(blk(p,:),X{p},tmp);  tmp = ops(tmp,'sym');
             schur(m+1,:) = schur(m+1,:) + ...
             [-blktrace(blk(p,:),A(p,:),tmp); blktrace(blk(p,:),C(p),tmp)]'; 
          end;
          schur_cpu(p) = cputime-tstart; 
       end;
    else;
       error(' user supplied schur routine is not allowed for AHOpredhlf '); 
    end;
    alpha = eps*norm(schur,'fro');
    schur = schur + spdiags(alpha*ones(m+1,1),0,m+1,m+1); 
%%
%%  rhs
%%  
    hRd = zeros(m+1,1);  hEinvRc = zeros(m+1,1);  EinvRc = cell(size(X));
    rc = sigma*mu-kap*tau;
    sigI = ops(ops(sigma*mu,'*',ONE),'spdiags',X);
    for p = 1:size(blk,1);
       if (schur_opt(p) == 1); 
          tmp = Prod2(blk(p,:),hX(p),ops(d(p),'spdiags',X(p))); 
          tmp = ops(tmp,'sym'); 
          EinvRc(p) = ops(Dsch(p),'.*',ops(sigI(p),'-',tmp)); 
          tmp = Prod3(blk(p,:),Vt(p),Rd(p),V(p),1); 
          tmp = Prod2(blk(p,:),hX(p),tmp); tmp = ops(tmp,'sym'); 
          tmp = ops(Dsch(p),'.*',tmp); 
          hRd = hRd + [blktrace(blk(p,:),B(p,:),tmp); -blktrace(blk(p,:),hC(p),tmp)];   
          hEinvRc = hEinvRc + ...
          [blktrace(blk(p,:),B(p,:),EinvRc(p)); -blktrace(blk(p,:),hC(p),EinvRc(p))];
       elseif (schur_opt(p) == 2);   
          tmp  = Prod2(blk(p,:),X(p),Z(p)); tmp = ops(tmp,'sym');  
          tmp  = Prod3(blk(p,:),Vt(p),ops(sigI(p),'-',tmp),V(p),1);
          EinvRc(p) = Prod3(blk(p,:),V(p),ops(Dsch(p),'.*',tmp),Vt(p),1); 
          tmp = Prod2(blk(p,:),X(p),Rd(p)); tmp = ops(tmp,'sym'); 
          tmp = Prod3(blk(p,:),Vt(p),tmp,V(p),1); 
          tmp = Prod3(blk(p,:),V(p),ops(Dsch(p),'.*',tmp),Vt(p),1,listRd(p),0);
          hRd = hRd + [blktrace(blk(p,:),A(p,:),tmp); -blktrace(blk(p,:),C(p),tmp)]; 
          hEinvRc = hEinvRc + ...
          [blktrace(blk(p,:),A(p,:),EinvRc(p)); -blktrace(blk(p,:),C(p),EinvRc(p))]; 
       end;
    end;
    rhs = eta*rp + [zeros(m,1); rc/tau] + eta*hRd - hEinvRc;   
%%
%% solve schur complement equation.
%%  
    ispd = 0; 
    [dy,dtau,Uchol,Lsch,Usch,Psch] = schurhlf(schur,rhs,b,kap,tau,mu,ispd); 
    clear schur; 
%%
%% compute (dX,dy,dZ)
%%
    dZ  = ops(ops(eta,'*',Rd),'-',ops(Asum(blk,A,dy,iscmpA),'-',ops(dtau,'*',C)));
    dX = cell(size(X)); 
    for p = 1:size(blk,1);
       if (schur_opt(p) == 1); 
          tmp = Prod3(blk(p,:),Vt(p),dZ(p),V(p),1); 
          tmp = Prod2(blk(p,:),hX(p),tmp); tmp = ops(tmp,'sym');  
          tmp = ops(EinvRc(p),'-',ops(Dsch(p),'.*',tmp)); 
          dX(p) = Prod3(blk(p,:),V(p),tmp,Vt(p),1);
       elseif (schur_opt(p) == 2); 
          tmp = Prod2(blk(p,:),X(p),dZ(p));  tmp = ops(tmp,'sym');
          tmp = Prod3(blk(p,:),Vt(p),tmp,V(p),1); 
          tmp = Prod3(blk(p,:),V(p),ops(Dsch(p),'.*',tmp),Vt(p),1); 
          dX(p) = ops(EinvRc(p),'-',tmp); 
       end;
    end;
    dkap = rc/tau - (kap/tau)*dtau; 
%%************************************************************************
