%%*************************************************************
%% Asum: compute the matrix 
%%
%%  Ay = Asum(blk,A,y,iscmp)
%%  
%%  input:  A = a CELL ARRAY with m columns. 
%%          y = mx1 vector.
%%          permAy = a permutation of [1:m] coding the order to 
%%                   sum the matrices y(k)*Ak, k = 1,...m. 
%%          iscmp = 1,  if Ay is complex
%%                  0,  otherwise.
%%                  
%%  output: Ay = sum_{k=1}^m y(k)*Ak, a column CELL ARRAY 
%%               with the same structure as A{:,1}. 
%%              
%% SDPT3: version 2.1
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last Modified: 7 Jul 99
%%*************************************************************

   function Ay = Asum(blk,A,y,iscmp)

   if (nargin == 3); iscmp = zeros(size(blk,1),1); end;

   Ay = mexAsum(blk,A,y,iscmp);    
   for p = 1:size(Ay,1); 
      tmp = Ay{p}; 
      [n,n2] = size(tmp);
      blktmp = blk{p,2};
      if strcmp(blk{p,1},'nondiag'); 
         if (length(blktmp) == 1)
            if (nnz(tmp) > 0.15*n*n);
               if issparse(tmp); tmp = full(tmp); end;
            else;
               if ~issparse(tmp); tmp = sparse(tmp); end;
            end;
         elseif (length(blktmp) > 1);
            if ~issparse(tmp); tmp = sparse(tmp); end;
         end;
      elseif strcmp(blk{p,1},'diag'); 
         if (nnz(tmp) > 0.15*n); 
            if issparse(tmp); tmp = full(tmp); end;
         else;
            if ~issparse(tmp); tmp = sparse(tmp); end;
         end;
      end;
      Ay{p} = tmp;
   end;
%%-------------------------------------------------------------



