%%***********************************************************************
%% GTpredhlf: Compute (dX,dy,dZdkap,dtau) for GT direction. 
%%
%% SDPT3: version 2.1 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 7 Jul 99
%%***********************************************************************

 function [dX,dy,dZ,dtau,dkap,hRd,ZH,invZH,d,sv2,phi2,psi2,Uchol,Lsch,Usch,Psch,schur_cpu] = GTpredhlf(blk,A,C,b,rp,Rd,sigma,mu,eta,schur_opt,schurfun_parms,iscmpA,normA,listA,permA,listRd,tau,kap,X,Zchol,invZchol,LA,TA,PA);

%%
%% compute the SVD of Xchol*Zchol' via the eigenvalue
%% decomposition of  Zchol*X*Zchol' = V*Sigma^2*V'
%%
    tmp = Prod3(blk,Zchol,X,ops(Zchol,'ctranspose'),4);
    [sv2,V] = blkeig(blk,tmp); 
    sv2 = ops(sv2,'real');  sv2 = ops(sv2,'+',eps*5*ops(sv2,'max'));
    ZH = Prod2(blk,ops(V,'ctranspose'),Zchol,21); 
    clear X  Zchol; 
    phi = ops(ZH,'row-norm');  phi2 = ops(phi,'.*',phi);
    ZH  = Prod2(blk,ops(ops(1,'./',phi),'spdiags',ZH),ZH);
    invZH = Prod2(blk,Prod2(blk,invZchol,V,31),ops(phi,'spdiags',ZH)); 
    clear V  invZchol; 
    invZHt = ops(invZH,'ctranspose');
    d = ops(1,'./',ops(invZHt,'row-norm'));
    dsqrt = ops(d,'sqrt'); 
    psi   = ops(ops(ops(sv2,'sqrt'),'./',phi),'./',d);  
    psi2 = ops(psi,'.*',psi);      
    DEinv = ops(ops(dsqrt,'.*',phi2),'rank1inv',ops(1,'./',dsqrt),blk);
    tmp  = ops(ops(ops(dsqrt,'.*',d),'.*',psi2),'rank1',dsqrt,blk);
    Dsch = ops(tmp,'.*',DEinv); 
%%
%% compute schur matrix
%%
    m = length(rp)-1;  hRd = []; 
    if isnumeric(schur_opt);
       schur = zeros(m+1);  
       for p = 1:size(blk,1); 
          tstart = cputime; 
          if (schur_opt(p) == 1);
             B = cell(1,m); 
             for k = 1:m
                if ~isempty(LA{p,k}); 
                   tmp = invZHt{p}(:,PA{p,k}); 
                   tmp = Prod2(blk(p,:),tmp,LA{p,k},22); 
                   tmp = Prod3(blk(p,:),tmp,TA{p,k},ops(tmp,'ctranspose'),1); 
                else
                   tmp = Prod3(blk(p,:),invZHt{p},A{p,k},invZH{p},1);
                end;
                B{k} = tmp;
                tmp = Dsch{p}.*tmp; 
                tmp2 = schur(1:k,k) + blktrace(blk(p,:),B(1:k),tmp);  
                schur(1:k,k) = tmp2;
                schur(k,1:k) = tmp2'; 
             end;
             clear B; 
          elseif (schur_opt(p) == 2); 
             list = [];
             for k = 1:m 
                permk = permA(p,k);
                if ~isstr(listA{p,k}); 
                   list = [list; listA{p,k}];
                else;
                   list = listA{p,k}; 
                end;   
                tmp = Prod3(blk(p,:),invZHt{p},A{p,permk},invZH{p},1); 
                tmp = Dsch{p}.*tmp; 
                tmp = Prod3(blk(p,:),invZH{p},tmp,invZHt{p},1,list,0); 
                idx = permA(p,1:k);
                tmp2 = schur(idx,permk) + blktrace(blk(p,:),A(p,idx),tmp);
                schur(idx,permk) = tmp2;
                schur(permk,idx) = tmp2'; 
             end;
          end;
          schur_cpu(p) = cputime-tstart; 
       end;
    else;
       eval(['[schur,hRd] = ',schur_opt,';']);  
       schur = schur ./ (normA*normA'); 
       if ~isempty(hRd);  hRd = hRd ./normA;  end;
    end;
    tmp = Prod3(blk,invZHt,C,invZH,1);
    tmp = Prod3(blk,invZH,ops(Dsch,'.*',tmp),invZHt,1,listRd,0);
    tmp2 = [-blktrace(blk,A,tmp); blktrace(blk,C,tmp)]; 
    schur(1:m+1,m+1) = tmp2;  
    schur(m+1,1:m+1) = tmp2'; 
    alpha = eps*norm(schur,'fro'); 
    schur = schur + spdiags(alpha*ones(m+1,1),0,m+1,m+1); 
%%
%% rhs
%%   
    rc = sigma*mu-kap*tau; 
    tmp = ops(ops(sigma*mu,'-',sv2),'spdiags',ZH);
    EinvRc = Prod3(blk,invZH,ops(DEinv,'.*',tmp),invZHt,1); 
    clear DEinv; 
    tmp = Prod3(blk,invZHt,Rd,invZH,1);  
    tmp = Prod3(blk,invZH,ops(Dsch,'.*',tmp),invZHt,1,listRd,0);
    hRd = [blktrace(blk,A,tmp); -blktrace(blk,C,tmp)];
    rhs = eta*rp + [zeros(m,1); rc/tau] + eta*hRd ...
          - [blktrace(blk,A,EinvRc);  -blktrace(blk,C,EinvRc)]; 
%%
%% solve schur complement equation. 
%%
    ispd = 1;
    [dy,dtau,Uchol,Lsch,Usch,Psch] = schurhlf(schur,rhs,b,kap,tau,mu,ispd); 
    clear schur; 
%%
%% compute (dy,dX,dZ)
%%   
    dZ  = ops(ops(eta,'*',Rd),'-',ops(Asum(blk,A,dy,iscmpA),'-',ops(dtau,'*',C)));
    tmp = Prod3(blk,invZHt,dZ,invZH,1);  tmp = ops(Dsch,'.*',tmp);  
    clear Dsch; 
    tmp = Prod3(blk,invZH,tmp,invZHt,1); 
    dX  = ops(EinvRc,'-',tmp); 
    dkap = rc/tau - (kap/tau)*dtau; 
%%********************************************************************

