%%*******************************************************************
%% HKMpred: Compute (dX,dy,dZ) for the H..K..M direction. 
%%
%% SDPT3: version 2.1 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 7 Jul 99
%%*******************************************************************

 function [dX,dy,dZ,hRd,Zinv,schur,indef,schur_cpu] = HKMpred(blk,A,rp,Rd,sigma,mu,schur_opt,schurfun_parms,iscmpA,normA,listA,permA,listRd,splistA,X,Xchol,invZchol);

%%
%% compute Zinv.
%%
    Zinv = Prod2(blk,invZchol, ops(invZchol,'ctranspose'), 221); 
%%
%% compute schur matrix
%%
    m = length(rp); hRd = []; 
    if isnumeric(schur_opt); 
       schur = zeros(m);
       for p = 1:size(blk,1);
          tstart = cputime; 
          if (schur_opt(p) == 1); 
             B = cell(1,m);
             for k = 1:m
                tmp = Prod3(blk(p,:),Xchol{p},A{p,k},invZchol{p},3); 
                B{k} = tmp; 
                %% matrix is nonsymmetric 
                tmp2 = schur(1:k,k) + blktrace(blk(p,:),B(1:k),tmp,0);
                schur(1:k,k) = tmp2; 
                schur(k,1:k) = tmp2'; 
             end; 
             clear B;
          elseif (schur_opt(p) == 2); 
           %%
           %% compute schur for matrices that are very sparse. 
           %% 
             J = 0;
             if (~iscmpA & strcmp(blk{p,1},'nondiag'));   
                for k = 1:m              
                   if isstr(splistA{p,k}); break; end; 
                   J = J + 1; 
                end;
                if (J > 0); 
                   if (length(blk{p,2}) == 1);
                      mexschur(blk(p,:),A(p,permA(p,:)),splistA(p,1:J),...
                      permA(p,:),full(X{p}),full(Zinv{p}),schur,1);
                   else
                      mexschur(blk(p,:),A(p,permA(p,:)),splistA(p,1:J),...
                      permA(p,:),sparse(X{p}),sparse(Zinv{p}),schur,1);
                   end;
                end;    
             end;
           %%
           %% compute schur for matrices that are not so sparse. 
           %% 
             list = []; 
             for k = 1:J 
                if ~isstr(listA{p,k}); 
                   list = [list; listA{p,k}];
                else;
                   list = listA{p,k}; 
                end;     
             end;
             for k = J+1:m
                if ~isstr(listA{p,k}); 
                   list = [list; listA{p,k}];
                else;
                   list = listA{p,k}; 
                end;      
                permk = permA(p,k);   
                tmp = Prod3(blk(p,:),Zinv{p},A{p,permk},X{p},0,list,1); 
                tmp = ops(tmp,'sym');
                idx = permA(p,1:k); 
                tmp2 = schur(idx,permk) + blktrace(blk(p,:),A(p,idx),tmp);
                schur(idx,permk) = tmp2; 
                schur(permk,idx) = tmp2'; 
             end;
          end;
          schur_cpu(p) = cputime-tstart;
       end;
    else;
       eval(['[schur,hRd] = ',schur_opt,';']);   
       tmp = spdiags(1./normA,0,m,m);
       schur = tmp*schur*tmp;       
       if ~isempty(hRd); hRd = hRd ./normA; end; 
    end;
    clear  Xchol invZchol; 
%%
%% compute r.h.s vector
%%
    if isnumeric(schur_opt) | isempty(hRd); 
       tmp = Prod3(blk,Zinv,Rd,X,0,listRd,1);  
       tmp = ops(tmp,'sym'); 
       hRd = blktrace(blk,A,tmp);
    end;
    EinvRc = ops(ops(sigma*mu,'*',Zinv),'-',X); 
    rhs = rp + hRd - blktrace(blk,A,EinvRc); 
%%
%% cholesky factorization 
%%
    pertdiag = 1e-16*sqrt(m)*norm(diag(schur)); 
    for k=1:m; 
       schur(k,k) = abs(schur(k,k))+pertdiag; 
    end;
    if issparse(schur), schur = full(schur); end;  
    [schur,indef] = chol(schur);       
    if indef; break; end; 
    dy = mexbsolve(schur,mexbsolve(schur,rhs,2),1); 
    if any(dy == inf); break; end;
%%    
    dZ  = ops(Rd,'-',Asum(blk,A,dy,iscmpA));  clear Rd; 
    tmp = Prod3(blk,Zinv,dZ,X);  clear X;
    tmp = ops(tmp,'sym');   
    dX  = ops(EinvRc,'-',tmp); 
%%=====================================================    

