%%************************************************************
%% HKMpredhlf: Compute H..K..M direction. 
%%
%% SDPT3: version 2.1 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 7 Jul 99
%%************************************************************

  function  [dX,dy,dZ,dtau,dkap,hRd,Zinv,Uchol,Lsch,Usch,Psch,schur_cpu] = HKMpredhlf(blk,A,C,b,rp,Rd,sigma,mu,eta,schur_opt,schurfun_parms,iscmpA,normA,listA,permA,listRd,splistA,tau,kap,X,Xchol,invZchol);

%%
%% compute Zinv, hRd := (Zinv x X) Rd. 
%%
    Zinv = Prod2(blk,invZchol, ops(invZchol,'ctranspose'), 221);
%%
%% compute schur matrix
%%
    m = length(rp)-1;  hRd = []; 
    if isnumeric(schur_opt); 
       schur = zeros(m+1);  
       for p = 1:size(blk,1); 
          tstart = cputime;
          if (schur_opt(p) == 1);
             B = cell(1,m);
             for k = 1:m
                tmp = Prod3(blk(p,:),Xchol{p},A{p,k},invZchol{p},3);
                B{k} = tmp;  
                %% matrix is nonsymmetric 
                tmp2 = schur(1:k,k) + blktrace(blk(p,:),B(1:k),tmp,0);
                schur(1:k,k) = tmp2;
                schur(k,1:k) = tmp2'; 
             end;
             clear B; 
          elseif (schur_opt(p) == 2); 
           %%
           %% compute schur for matrices that are very sparse. 
           %% 
             J = 0;
             if (~iscmpA);   
                for k = 1:m              
                   if isstr(splistA{p,k}); break; end; 
                   J = J + 1; 
                end;
                if (J > 0); 
                   if (length(blk{p,2}) == 1);
                      mexschur(blk(p,:),A(p,permA(p,:)),splistA(p,1:J),...
                      permA(p,:),full(X{p}),full(Zinv{p}),schur,1);
                   else
                      mexschur(blk(p,:),A(p,permA(p,:)),splistA(p,1:J),...
                      permA(p,:),sparse(X{p}),sparse(Zinv{p}),schur,1);
                   end;
                end;    
             end;
           %%
           %% compute schur for matrices that are not so sparse. 
           %% 
             list = [];
             for k = 1:J 
                if ~isstr(listA{p,k}); 
                   list = [list; listA{p,k}];
                else;
                   list = listA{p,k}; 
                end;     
             end;
             for k = J+1:m
                permk = permA(p,k);
                if ~isstr(listA{p,k}); 
                   list = [list; listA{p,k}];
                else;
                   list = listA{p,k}; 
                end;   
                tmp = Prod3(blk(p,:),Zinv{p},A{p,permk},X{p},0,list,1); 
                tmp = ops(tmp,'sym');
                idx = permA(p,1:k); 
                tmp2 = schur(idx,permk) + blktrace(blk(p,:),A(p,idx),tmp); 
                schur(idx,permk) = tmp2; 
                schur(permk,idx) = tmp2'; 
             end;
          end;
          schur_cpu(p) = cputime-tstart; 
       end;
    else;
       eval(['[schur,hRd] = ',schur_opt,';']);         
       schur = schur ./ (normA*normA');   
       if ~isempty(hRd);  hRd = hRd ./ normA; end; 
    end;
    clear  Xchol invZchol;
    tmp  = Prod3(blk,Zinv,C,X,0,listRd,1); 
    tmp = ops(tmp,'sym');
    tmp2 = [-blktrace(blk,A,tmp); blktrace(blk,C,tmp)]; 
    schur(1:m+1,m+1) = tmp2; 
    schur(m+1,1:m+1) = tmp2'; 
    alpha = eps*norm(schur,'fro');
    schur = schur + spdiags(alpha*ones(m+1,1),0,m+1,m+1); 
%%  
%% rhs 
%%
    rc = sigma*mu -kap*tau;
    tmp = Prod3(blk,Zinv,Rd,X,0,listRd,1); tmp = ops(tmp,'sym');
    if isnumeric(schur_opt) | isempty(hRd);  
       hRd = [blktrace(blk,A,tmp); -blktrace(blk,C,tmp)];
    else;
       hRd(m+1) = -blktrace(blk,C,tmp); 
    end;
    EinvRc = ops(ops(sigma*mu,'*',Zinv),'-',X); 
    rhs = eta*rp + [zeros(m,1); rc/tau] + eta*hRd ...
          - [blktrace(blk,A,EinvRc); -blktrace(blk,C,EinvRc)];  
%%
%% solve schur complement equation. 
%%
    ispd = 1;
    [dy,dtau,Uchol,Lsch,Usch,Psch] = schurhlf(schur,rhs,b,kap,tau,mu,ispd); 
    clear schur; 
%%
%% compute (dX,dy,dZ)
%%
    dZ  = ops(ops(eta,'*',Rd),'-',ops(Asum(blk,A,dy,iscmpA),'-',ops(dtau,'*',C)));
    tmp  = Prod3(blk,Zinv,dZ,X);   tmp = ops(tmp,'sym'); 
    dX   = ops(EinvRc,'-',tmp); 
    dkap = rc/tau - (kap/tau)*dtau;
%%=====================================================      
