%%**********************************************************************
%% NTpred: Compute (dX,dy,dZ) for NT direction. 
%%                       
%% SDPT3: version 2.1 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 7 Jul 99
%%**********************************************************************

 function [dX,dy,dZ,hRd,W,G,sv,schur,indef,schur_cpu] = NTpred(blk,A,rp,Rd,sigma,mu,schur_opt,schurfun_parms,iscmpA,normA,listA,permA,listRd,splistA,X,Zchol,invZchol,AR);
                       
%%
%% compute SVD of Xchol*Zchol via eigenvalue decompostion of
%%     Zchol * X * Zchol' = V * diag(sv2) * V'. 
%% compute W, where W satisfies W*Z*W = X. 
%%     W = G'*G,  where G = diag(sqrt(sv)) * (invZchol*V)'
%%
    tmp = Prod3(blk,Zchol,X,ops(Zchol,'ctranspose'),4); 
    clear X; 
    [sv2,V] = blkeig(blk,tmp); 
    sv2 = ops(sv2,'real');  sv2 = ops(sv2,'+',eps*5*ops(sv2,'max'));
    sv  = ops(sv2,'sqrt'); 
    G = Prod2(blk,invZchol,V,31); 
    G = Prod2(blk,ops(ops(sv,'sqrt'),'spdiags',G),ops(G,'ctranspose'));
    clear Zchol  invZchol  V; 
    W = Prod2(blk,ops(G,'ctranspose'),G,1);  
%%
%% compute schur matrix
%%
    m = length(rp); hRd = []; 
    if isnumeric(schur_opt); 
       schur = zeros(m);   
       for p = 1:size(blk,1);   
          tstart = cputime;
          if (schur_opt(p) == 1); 
             B = cell(1,m); 
             Wchol = blkchol(blk(p,:),W{p}); 
             Wcholt = ops(Wchol,'ctranspose'); 
             for k = 1:m
                if ~isempty(AR{p,k});
                   tmp = Prod3(blk(p,:),Wchol,AR{p,k},Wcholt,2); 
                   tmp = ops(tmp,'+',ops(tmp,'ctranspose'));%%do not use 'sym' 
                   B{k} = tmp;  
                else; 
                   tmp = Prod3(blk(p,:),Wchol,A{p,k},Wcholt,4);
                   B{k} = tmp; 
                end;
                tmp2 = schur(1:k,k) + blktrace(blk(p,:),B(1:k),tmp);
                schur(1:k,k) = tmp2; 
                schur(k,1:k) = tmp2'; 
             end;
             clear B Wchol Wcholt; 
          elseif (schur_opt(p) == 2);
           %%
           %% compute schur for matrices that are very sparse. 
           %% 
             J = 0; 
             if (~iscmpA & strcmp(blk{p,1},'nondiag'));
                for k = 1:m              
                   if isstr(splistA{p,k}); break; end; 
                   J = J + 1; 
                end;
                if (J > 0);
                   if (length(blk{p,2}) == 1); 
                      mexschur(blk(p,:),A(p,permA(p,:)),splistA(p,1:J),...
                      permA(p,:),full(W{p}),full(W{p}),schur);
                   else
                      mexschur(blk(p,:),A(p,permA(p,:)),splistA(p,1:J),...
                      permA(p,:),sparse(W{p}),sparse(W{p}),schur);
                   end;
                end;
             end;
           %%
           %% compute schur for matrices that are not so sparse. 
           %% 
             list = []; 
             for k = 1:J 
                if ~isstr(listA{p,k}); 
                   list = [list; listA{p,k}];
                else;
                   list = listA{p,k}; 
                end;     
             end;
             for k = J+1:m
                if ~isstr(listA{p,k}); 
                   list = [list; listA{p,k}];
                else
                   list = listA{p,k}; 
                end;      
                permk = permA(p,k);  
                tmp = Prod3(blk(p,:),W{p},A{p,permk},W{p},1,list,1);                    
                idx = permA(p,1:k);
                tmp2 = schur(idx,permk) + blktrace(blk(p,:),A(p,idx),tmp);
                schur(idx,permk) = tmp2; 
                schur(permk,idx) = tmp2'; 
             end;
          end;
          schur_cpu(p) = cputime-tstart; 
       end; 
    else;
       eval(['[schur,hRd] = ',schur_opt,';']);  
       tmp = spdiags(1./normA,0,m,m);
       schur = tmp*schur*tmp;       
       if ~isempty(hRd);  hRd = hRd ./ normA;  end; 
    end;
%%
%% compute r.h.s vector
%%
    if isnumeric(schur_opt) | isempty(hRd); 
       tmp = Prod3(blk,W,Rd,W,1,listRd,1);
       hRd = blktrace(blk,A,tmp); 
    end;
    tmp = ops( ops(ops(sigma*mu,'./',sv),'-',sv), 'spdiags', G );
    EinvRc = Prod3(blk,ops(G,'ctranspose'),tmp,G,1); 
    rhs = rp + hRd - blktrace(blk,A,EinvRc); 
%%
%% Cholesky factorization
%%
    pertdiag = 1e-16*sqrt(m)*norm(diag(schur)); 
    for k=1:m; 
       schur(k,k) = abs(schur(k,k))+pertdiag; 
    end;
    if issparse(schur), schur = full(schur); end; 
    [schur,indef] = chol(schur); 
    if indef; break; end; 
    dy = mexbsolve(schur,mexbsolve(schur,rhs,2),1); 
    if any(dy == inf); break; end; 
%%
    dZ  = ops(Rd,'-',Asum(blk,A,dy,iscmpA)); 
    tmp = Prod3(blk,W,dZ,W,1); 
    dX  = ops(EinvRc,'-',tmp); 
%%======================================================================

