%%**********************************************************************
%% NTpredhlf: Compute (dX,dy,dZ,dkap,dtau) for NT direction. 
%%
%% SDPT3: version 2.1 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 7 Jul 99
%%**********************************************************************

 function [dX,dy,dZ,dtau,dkap,hRd,W,G,sv,Uchol,Lsch,Usch,Psch,schur_cpu] = NTpredhlf(blk,A,C,b,rp,Rd,sigma,mu,eta,schur_opt,schurfun_parms,iscmpA,normA,listA,permA,listRd,splistA,tau,kap,X,Zchol,invZchol,AR);

%%
%% compute SVD of Xchol*Zchol via eigenvalue decompostion of
%%     Zchol * X * Zchol' = V * diag(sv2) * V'. 
%% compute W, where W satisfies W*Z*W = X. 
%%     W = G'*G,  where G = diag(sqrt(sv)) * (invZchol*V)'
%%
    tmp = Prod3(blk,Zchol,X,ops(Zchol,'ctranspose'),4);
    clear X; 
    [sv2,V] = blkeig(blk,tmp); 
    sv2 = ops(sv2,'real');  sv2 = ops(sv2,'+',eps*5*ops(sv2,'max'));
    sv  = ops(sv2,'sqrt'); 
    G = Prod2(blk,invZchol,V,31); 
    G = Prod2(blk,ops(ops(sv,'sqrt'),'spdiags',G),ops(G,'ctranspose'));
    clear Zchol  invZchol  V;  
    W = Prod2(blk,ops(G,'ctranspose'),G,1);  
%%
%% compute schur matrix
%%
    m = length(rp)-1;  hRd = [];
    if isnumeric(schur_opt); 
       schur = zeros(m+1);     
       for p = 1:size(blk,1);
          tstart = cputime; 
          if (schur_opt(p) == 1);  
             B = cell(1,m);      
             Wchol = blkchol(blk(p,:),W{p}); 
             Wcholt = ops(Wchol,'ctranspose'); 
             for k = 1:m
                if ~isempty(AR{p,k});
                   tmp = Prod3(blk(p,:),Wchol,AR{p,k},Wcholt,2); 
                   tmp = ops(tmp,'+',ops(tmp,'ctranspose')); 
                   B{k} = tmp; 
                else; 
                   tmp = Prod3(blk(p,:),Wchol,A{p,k},Wcholt,4);
                   B{k} = tmp;
                end;
                tmp2 = schur(1:k,k) + blktrace(blk(p,:),B(1:k),tmp);
                schur(1:k,k) = tmp2;
                schur(k,1:k) = tmp2';
             end;
             clear B Wchol Wcholt; 
          elseif (schur_opt(p) == 2);
           %%
           %% compute schur for matrices that are very sparse. 
           %% 
             J = 0; 
             if (~iscmpA); 
                for k = 1:m              
                   if isstr(splistA{p,k}); break; end; 
                   J = J + 1; 
                end;
                if (J > 0);
                   if (length(blk{p,2}) == 1); 
                      mexschur(blk(p,:),A(p,permA(p,:)),splistA(p,1:J),...
                      permA(p,:),full(W{p}),full(W{p}),schur);
                   else
                      mexschur(blk(p,:),A(p,permA(p,:)),splistA(p,1:J),...
                      permA(p,:),sparse(W{p}),sparse(W{p}),schur);
                   end;
                end;
             end;
           %%
           %% compute schur for matrices that are not so sparse. 
           %% 
             list = []; 
             for k = 1:J 
                if ~isstr(listA{p,k}); 
                   list = [list; listA{p,k}];
                else;
                   list = listA{p,k}; 
                end;     
             end; 
             for k = J+1:m
                permk = permA(p,k);
                if ~isstr(listA{p,k}); 
                   list = [list; listA{p,k}];
                else;
                   list = listA{p,k}; 
                end;                   
                tmp = Prod3(blk(p,:),W{p},A{p,permk},W{p},1,list,1); 
                idx = permA(p,1:k); 
                tmp2 = schur(idx,permk) + blktrace(blk(p,:),A(p,idx),tmp); 
                schur(idx,permk) = tmp2;
                schur(permk,idx) = tmp2'; 
             end;
          end;
          schur_cpu(p) = cputime-tstart; 
       end;
    else;
       eval(['[schur,hRd] = ',schur_opt,';']);  
       schur = schur ./ (normA*normA'); 
       if ~isempty(hRd);  hRd = hRd ./ normA;  end;
    end;
    tmp = Prod3(blk,W,C,W,1,listRd,1); 
    tmp2 = [-blktrace(blk,A,tmp); blktrace(blk,C,tmp)];   
    schur(1:m+1,m+1) = tmp2;
    schur(m+1,1:m+1) = tmp2'; 
    alpha = eps*norm(schur,'fro');
    schur = schur + spdiags(alpha*ones(m+1,1),0,m+1,m+1); 
%%
%% rhs
%%
    rc = sigma*mu-kap*tau;    
    tmp = Prod3(blk,W,Rd,W,1,listRd,1);  
    if isnumeric(schur_opt) | isempty(hRd); 
       hRd = [blktrace(blk,A,tmp); -blktrace(blk,C,tmp)];
    else;
       hRd(m+1) = -blktrace(blk,C,tmp);
    end;
    tmp = ops( ops(ops(sigma*mu,'./',sv),'-',sv), 'spdiags', G );
    EinvRc = Prod3(blk,ops(G,'ctranspose'),tmp,G,1);
    rhs = eta*rp + [zeros(m,1); rc/tau] + eta*hRd ...
          - [blktrace(blk,A,EinvRc);  -blktrace(blk,C,EinvRc)]; 
%%
%% solve schur complement equation. 
%%
    ispd = 1;
    [dy,dtau,Uchol,Lsch,Usch,Psch] = schurhlf(schur,rhs,b,kap,tau,mu,ispd); 
    clear schur; 
%%
%% compute (dy,dX,dZ)
%%   
    dZ  = ops(ops(eta,'*',Rd),'-',ops(Asum(blk,A,dy,iscmpA),'-',ops(dtau,'*',C)));
    tmp = Prod3(blk,W,dZ,W,1); 
    dX = ops(EinvRc,'-',tmp);
    dkap = rc/tau - (kap/tau)*dtau; 
%%**********************************************************************
  

