%%*******************************************************************
%% Prod2: compute the block diagonal matrix A*B
%%
%%    C = Prod2(blk,A,B,options);
%%
%% INPUT:  blk = a cell array describing the block structure of A and B
%%         A,B= square matrices or column vectors, or single column
%%              CELL ARRAYS consisting of square matrices or 
%%              column vectors. 
%%
%%   options = 0     if no special structure 
%%             1     if                               C is hermitian
%%             21    if        B is upper triangular 
%%             211   if        B is upper triangular, C is hermitian
%%             22    if        B is lower triangular
%%             221   if        B is lower triangular, C is hermitian
%%             31    if A is upper triangular
%%             311   if A is upper triangular,        C is hermitian
%%             32    if A is lower triangular
%%             321   if A is lower triangular,        C is hermitian
%%             41    if A is upper triangular, B is upper triangular
%%             42    if A is upper triangular, B is lower triangualr
%%             421   if A is upper triangular, B is lower triangualr
%%                                                    C is hermitian
%%
%% SDPT3: version 2.1 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 7 Jul 99
%%*******************************************************************

   function [C] = Prod2(blk,A,B,options);

   if (nargin == 3); options = 0; end; 

   spdensity = 0.5;   
 
   if (~iscell(A) & ~iscell(B));
      [m,n] = size(A);   
      if (n > 1) & (m==n); %% A,B are square matrices. 
         isspA = issparse(A); isspB = issparse(B); 
         isrealA = isreal(A); isrealB = isreal(B); 
         if (options == 0 & ~isspA & ~isspB) | ...
            (length(blk{1,2}) == 1 & isspA & isspB); 
            C = A*B; 
         else 
            C = mexProd2(blk,A,B,options);
            if (isspA & isspB); 
               if (options == 1 | options == 211 | options == 221 | ...
                  options == 311 | options == 321 | options == 421); 
                  C = spdiags(real(diag(C)),0,m,n) + triu(C,1) + triu(C,1)';
               end;
            end;
         end;
         if (nnz(C) <= spdensity*n*n);
            if ~issparse(C); C = sparse(C); end;
         else;
            if issparse(C); C = full(C); end; 
         end;
      else  %% A,B are real column vectors.
         C = A.*B;
      end; 
   elseif (iscell(A) & iscell(B));
      C = cell(size(A)); 
      for p = 1:size(A,1); 
         C{p} = Prod2(blk(p,:),A{p},B{p},options); 
      end; 
   else
      error(['Prod2: A,B are different objects']); 
   end;  
%%===================================================================

