%%*******************************************************************
%% Prod3: compute block diagonal matrix products
%%
%%    Q = Prod3(blk,A,B,C,options,list,symA), where Q = A*B*C. 
%%
%% INPUT:  A,B,C = square matrices or column vectors, or single
%%                 column CELL ARRAYS consisting of square matrices 
%%                 or column vectors. 
%%
%%       options = 0  if no special structure, 
%%               = 1  if C = A', and B is hermitian
%%               = 2  if A, B are upper triangular, C is lower triangular
%%               = 3  if A, C are upper triangular
%%               = 4  if C = A', A upper triangular, and B is hermitian
%%               = 5  if C = A', A lower triangular, and B is hermitian
%%
%%        list = each row of list carrys the 
%%               indices of elements of Q to be computed. 
%%
%%        symA = 0, if A is non-heritian [default]. 
%%             = 1, if A is hermitian. 
%%
%% SDPT3: version 2.1 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 7 Jul 99
%%*******************************************************************
 
   function Q = Prod3(blk,A,B,C,options,list,symA);

   if (nargin <= 6); symA = 0; end; 
   if (nargin <= 5); 
      if iscell(A); 
         for k = 1:size(A,1); list{k} = 'full'; end;
      else;
         list = 'full';
      end;
   end; 
   if (nargin <= 4); options = 0; end; 
%%
   if ~(iscell(A) | iscell(B) | iscell(C) | iscell(list));
      [m,n] = size(A);
      if strcmp(list,'full'); 
         if (options == 0); 
            Q = Prod2(blk, Prod2(blk,A,B,0), C, 0); 
         elseif (options == 1);
            Q = Prod2(blk, Prod2(blk,A,B,0), C, 1);
         elseif (options == 2); 
            Q = Prod2(blk, Prod2(blk,A,B,41), C, 42);    
         elseif (options == 3); 
            Q = Prod2(blk, Prod2(blk,A,B,31), C, 21);
         elseif (options == 4);
            Q = Prod2(blk, A, Prod2(blk,B,C,22), 311);
         elseif (options == 5);
            Q = Prod2(blk, A, Prod2(blk,B,C,21), 321); 
         end;
      elseif ~isempty(list);
         if strcmp(blk{1,1},'nondiag'); 
            tmp = Prod2(blk,B,C,0); 
            if (options == 1 | options == 4 | options == 5); 
               symQ = 1; 
            else
               symQ = 0; 
            end;      
            %%
            %% IMPORTANT: mexProd2nz computes A'*tmp
            %%          
            list = sortrows(list,[2 1]);    
            if symA; %% A is hermitian
               Q = mexProd2nz(blk,A,tmp,list,symQ);  
            else;
               Q = mexProd2nz(blk,A',tmp,list,symQ);   
            end; 
            if symQ; Q = Q + triu(Q,1)'; end;      
         elseif strcmp(blk{1,1},'diag'); 
            Q = A.*B.*C; 
         end;
      else 
         Q = sparse(m,n); 
      end;
   elseif (iscell(A) & iscell(B) & iscell(C) & iscell(list)); 
      Q = cell(size(A)); 
      for p = 1:size(blk,1);  
         Q{p} = Prod3(blk(p,:),A{p},B{p},C{p},options,list{p},symA); 
      end;
   else
      error('Prod3: input must all be cell arrays or all matrices'); 
   end 
%%===================================================================

