%%******************************************************************
%% aasen: decompose a nxn symmetric matrix A by using the 
%%        Aasen's method.
%%
%%        [L,T,piv] = aasen(A); 
%%
%%        A(pinv,piv) = L*T*L'  where 
%%
%%        L is a lower triangular matrix, 
%%        T is a symmetric tridiagonal matrix. 
%%        piv is a permutation of [1:n]. 
%%
%% Suppose P = eye(n); P = P(piv,:). Then 
%%
%%             P*A*P' = L*T*L' or  A = P'*L*T*L'*P. 
%%
%% Note that   P*A = A(piv,:);  A*P' = A(:,piv). 
%%         
%% SDPT3: version 2.1 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last Modified: 7 Jul 99
%%******************************************************************

   function [L,T,P] = aasen(A); 

   if ~iscell(A);
      [m,n] = size(A);
      if (n > 1) & (~issparse(A)); 
         [L,alp,beta,P] = mexaasen(A); 
         beta1 = [beta; 0]; beta2 = [0; conj(beta)]; 
         T = spdiags([beta1 alp beta2],[-1:1],n,n);
      else; 
         L = []; T = []; P = []; 
      end;
   else; 
      L = cell(size(A)); T = cell(size(A)); P = cell(size(A)); 
      for p = 1:size(A,1); 
          for k = 1:size(A,2); 
              [L{p,k},T{p,k},P{p,k}] = aasen(A{p,k}); 
          end; 
      end;
   end;
%%==================================================================
