%%********************************************************************
%% infeaspt: generate an initial point for sdp.m 
%%
%%  [X0,y0,Z0] = infeaspt(blk,A,C,b,options,scalefac);
%%
%%  options = 1  if want X0,Z0 to be scaled identity matrices
%%          = 2  if want X0,Z0 to be scalefac*(identity matrices). 
%%
%% SDPT3: version 2.1 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last Modified: 7 Jul 99
%%********************************************************************

   function [X0,y0,Z0] = infeaspt(blk,A,C,b,options,scalefac); 
%%
   if (nargin < 5); options = 1; end; 
   if (options == 1); scalefac = []; end; 
   if (options == 2) & (nargin < 6); scalefac = 1000; end; 
   if (scalefac < 0); error('scalefac must a non negative number'); end;
%% 
   if ~iscell(C);
      blktmp = blk{2};  
      m = length(b); n = length(C);  
      y0 = zeros(m,1);
      b2 = 1+abs(b');
      if (options == 1);   
         if ~strcmp(blk{1},'diag'); 
            s = [0 cumsum(blktmp)];
            isspA = 1; 
            for k = 1:size(A,2); 
               Ak = A{k}; 
               isspA = min(isspA,issparse(Ak));
               for i = 1:length(blktmp)
                  pos = [s(i)+1:s(i+1)];
                  tmp = Ak(pos,pos);
                  normA(i,k) = 1+sqrt(sum(sum(tmp.*conj(tmp))));  
               end;
            end; 
            if isspA & issparse(C); X0 = sparse(n,n); Z0 = sparse(n,n); 
            else;                   X0 = zeros(n);    Z0 = zeros(n); 
            end; 
            for i = 1:length(blktmp)
               pos = [s(i)+1:s(i+1)];  ni = length(pos);
               tmp = C(pos,pos);
               normC(i) = 1+sqrt(sum(sum(tmp.*conj(tmp))));
               X0(pos,pos) = ni *max(b2./normA(i,:)) *speye(ni);
               Z0(pos,pos) = (speye(ni)/sqrt(ni))* ...
                             max([normA(i,:), normC(i)]);
            end;
         elseif strcmp(blk{1},'diag'); 
            X0 = zeros(n,1); Z0 = zeros(n,1);  
            for k = 1:size(A,2); 
               Ak = A{k};
               normA(:,k) = 1+abs(Ak);            
            end;
            for i = 1:n;
               X0(i) = max(b2./normA(i,:));
               Z0(i) = max([normA(i,:), 1+abs(C(i))]); 
            end;
         end;
      elseif (options == 2); 
         if ~strcmp(blk{1},'diag'); 
            X0 = scalefac*speye(sum(blktmp)); 
            Z0 = scalefac*speye(sum(blktmp));  
         elseif strcmp(blk{1},'diag');
            X0 = scalefac*ones(blktmp,1);     
            Z0 = scalefac*ones(blktmp,1);
         end;
      end;
   else; 
      X0 = cell(size(C)); Z0 = cell(size(C)); 
      for p = 1:size(blk,1); 
          [X0{p},y0,Z0{p}] = infeaspt(blk(p,:),A(p,:),C{p},b,options,scalefac);
      end;
   end; 
%%====================================================================  

