/**********************************************************
* mexaasen.c:  C-MEX file for aasen.m 
*
*      [L,alp,b,piv] = mexaasen(A); 
*
*   Let T = diag(alp) + diag(b,-1) + diag(conj(b),1), then
*  
*       A(piv,piv) = L*T*L'  
*     
*
**********************************************************/

#include <math.h>
#include "mex.h"

/********************************************
* find the maximum of a real vector
*********************************************/
int maxabs(double *v, int j, int n)

{ double big, temp;
  int k, idx; 

  big = 0.0; 
  for (k=j+1; k<=n-1; k++) {
      temp = fabs(v[k]); 
      if (temp >= big) {
         big = temp; 
         idx = k; 
      }
  }  
  return idx;   
}
/********************************************
* A real
*********************************************/
static void assen_real(int n, double *A, double *L, double *alp, 
                       double *b, double *piv, double *h, 
                       double *v, double *l)

{ int i, j, k, kn, idx; 
  double t, sw;

  for (k=0; k<=n-1; k++) { 
      piv[k] = k+1; 
      L[k+k*n] = 1; 
  }

  for (j=0; j<=n-1; j++) { 
      if (j==0) {
          h[0] = A[0]; }
      else if (j == 1) {
          h[0] = b[0];  h[1] = A[1+n]; }
      else { 
          l[0] = 0; l[j] = 1; 
          for (k=1; k<=j-1; k++) {
              l[k] = L[j+k*n]; 
          }
          h[j] = A[j+j*n]; 
          for (k=0; k<=j-1; k++) {
  	      if (k>=1) {       
  	         h[k] = b[k-1]*l[k-1] + alp[k]*l[k] + b[k]*l[k+1]; }
              else if (k==0) {
 	         h[k] = alp[k]*l[k] + b[k]*l[k+1]; }  
              h[j] = h[j] - l[k]*h[k];
          }
      }
      if (j == 0 || j == 1) { 
  	 alp[j] = h[j]; }
      else { 
	 alp[j] = h[j] - b[j-1]*L[j+(j-1)*n]; }
      if (j<=n-2) {
	 for (k=j+1; k<=n-1; k++) {
             t = 0; 
             for (i=0; i <=j; i++) {t += L[k+i*n]*h[i]; } 
             v[k] = A[k+j*n] - t;
         } 
	 idx = maxabs(v,j,n); 
         sw = v[idx]; v[idx] = v[j+1]; v[j+1] = sw; 
         for (k=1; k<=j; k++){
             kn = k*n; 
             sw = L[idx+kn]; L[idx+kn] = L[j+1+kn]; L[j+1+kn] = sw;
	 }
         for (k=j+1; k<=n-1; k++){
             kn = k*n; 
             sw = A[idx+kn]; A[idx+kn] = A[j+1+kn]; A[j+1+kn] = sw;
         }
         for (k=j+1; k<=n-1; k++){
             sw = A[k+idx*n]; A[k+idx*n] = A[k+(j+1)*n]; A[k+(j+1)*n] = sw;
         } 
         b[j] = v[j+1]; 
         sw = piv[idx]; piv[idx] = piv[j+1]; piv[j+1]= sw;
      }
      if (j <=n-3) {
         t = v[j+1]; 
         if (t!=0) {
	    for (k=j+2; k<n; k++) {
                L[k+(j+1)*n] = v[k]/t; 
            }
         }
         else {
  	    for (k=j+2; k<n; k++) {
                L[k+(j+1)*n] = v[k];
            } 
         }
      }  
  }
  return;
}
/********************************************
* find the maximum of a complex vector
*********************************************/
int maxabs_cmp(double *v, double *vi, int j, int n)

{ double big, temp;
  int k, idx; 

  big = 0.0; 
  for (k=j+1; k<=n-1; k++) {
      temp = sqrt(v[k]*v[k]+vi[k]*vi[k]); 
      if (temp >= big) {
         big = temp; 
         idx = k; 
      }
  }    
  return idx;   
}
/********************************************
* A complex
*********************************************/
static void assen_cmp(int n, double *A, double *L, double *alp, 
                      double *b, double *piv, double *h, 
                      double *v, double *l,
                      double *Ai, double *Li, double *bi,
                      double *hi, double *vi, double *li)

{ int i, j, k, kn, idx; 
  double t, ti, abst, sw, swi; 

  for (k=0; k<=n-1; k++) { 
      piv[k] = k+1; 
      L[k+k*n] = 1; 
  }
  for (j=0; j<=n-1; j++) { 
      if (j==0) {
          h[0] = A[0]; hi[0] = Ai[0]; }
      else if (j == 1) {
          h[0] = b[0]; hi[0] = bi[0]; 
          h[1] = A[1+n];  hi[1] = Ai[1+n]; }
      else { 
          l[0] = 0; l[j] = 1; 
          for (k=1; k<=j-1; k++) {
              l[k] = L[j+k*n];  li[k] = -Li[j+k*n]; 
          }
          h[j] = A[j+j*n];  hi[j] = Ai[j+j*n];
          for (k=0; k<=j-1; k++) {
  	      if (k>=1) {   
		 h[k]  = (b[k-1]*l[k-1]-bi[k-1]*li[k-1]); 
                 h[k] += alp[k]*l[k] + (b[k]*l[k+1]+bi[k]*li[k+1]); 
		 hi[k]  = (b[k-1]*li[k-1]+bi[k-1]*l[k-1]);
		 hi[k] += alp[k]*li[k] + (b[k]*li[k+1]-bi[k]*l[k+1]); 
              }
              else if (k==0) {
 	         h[k]  = alp[k]*l[k] + (b[k]*l[k+1]+bi[k]*li[k+1]); 
		 hi[k] = alp[k]*li[k] + (b[k]*li[k+1]-bi[k]*l[k+1]); 
              }  
              h[j]  = h[j] - (l[k]*h[k] + li[k]*hi[k]);
              hi[j] = hi[j] - (l[k]*hi[k] - li[k]*h[k]);
          }
      }
      if (j == 0 || j == 1) { 
  	 alp[j] = h[j]; }
      else { 
	 alp[j] = h[j] - (b[j-1]*L[j+(j-1)*n] + bi[j-1]*Li[j+(j-1)*n]); }
      if (j<=n-2) {
	 for (k=j+1; k<=n-1; k++) {
             t = 0;  ti = 0;  
             for (i=0; i <=j; i++) {
                 t  += L[k+i*n]*h[i] - Li[k+i*n]*hi[i]; 
                 ti += L[k+i*n]*hi[i] + Li[k+i*n]*h[i]; } 
             v[k]  = A[k+j*n] - t;
             vi[k] = Ai[k+j*n] - ti;
         } 
	 idx = maxabs_cmp(v,vi,j,n); 
         sw = v[idx]; swi = vi[idx]; 
         v[idx] =v[j+1]; vi[idx] = vi[j+1]; 
         v[j+1] = sw;    vi[j+1] = swi; 
         for (k=1; k<=j; k++){
             kn = k*n; 
             sw = L[idx+kn];          swi = Li[idx+kn];
             L[idx+kn] = L[j+1+kn];   Li[idx+kn] = Li[j+1+kn]; 
             L[j+1+kn] = sw;          Li[j+1+kn] = swi; 
	 }
         for (k=j+1; k<=n-1; k++){
             kn = k*n; 
             sw = A[idx+kn];          swi = Ai[idx+kn];
             A[idx+kn] = A[j+1+kn];   Ai[idx+kn] = Ai[j+1+kn]; 
             A[j+1+kn] = sw;          Ai[j+1+kn] = swi; 
         }
         for (k=j+1; k<=n-1; k++){
             sw = A[k+idx*n];            swi = Ai[k+idx*n]; 
             A[k+idx*n] = A[k+(j+1)*n];  Ai[k+idx*n] = Ai[k+(j+1)*n];
             A[k+(j+1)*n] = sw;          Ai[k+(j+1)*n] = swi; 
         } 
         b[j] = v[j+1]; bi[j] = vi[j+1]; 
         sw = piv[idx]; 
         piv[idx] = piv[j+1]; 
         piv[j+1] = sw;
      }
      if (j <=n-3) {
         t = v[j+1]; ti = vi[j+1]; 
         abst = sqrt(t*t+ti*ti); 
         t = t/abst; ti = ti/abst; 
         if (abst!=0) {
	    for (k=j+2; k<n; k++) {
                L[k+(j+1)*n]  = (v[k]*t + vi[k]*ti)/abst; 
                Li[k+(j+1)*n] = (-v[k]*ti + vi[k]*t)/abst;
            }
         }
         else {
  	    for (k=j+2; k<n; k++) {
                L[k+(j+1)*n] = v[k];  Li[k+(j+1)*n] = vi[k];
            } 
         }
      }  
  }
  return;
}
/**********************************************************/

void mexFunction(
      int nlhs,   mxArray  *plhs[], 
      int nrhs,   const mxArray  *prhs[] )
{
   mxArray  *workarray[3]; 
   mxArray  *array_ptr; 
   double   *A,  *L,  *b, *alp, *piv, *h,  *v,  *l; 
   double   *Ai, *Li, *bi,            *hi, *vi, *li; 
   int      m1, n1, iscmpA; 

/* Check for proper number of arguments */
   if (nrhs!=1){
     mexErrMsgTxt("ProdT: requires 1 input arguments."); }

/* get matrix dimension */

   m1 = mxGetM(prhs[0]);
   n1 = mxGetN(prhs[0]); 
   if (m1!=n1){
      mexErrMsgTxt("aasen: input matrix must be square."); }

/* Create a matrix for the return argument */

   iscmpA = mxIsComplex(prhs[0]); 
   if (!iscmpA) { 
      array_ptr =  mxCreateDoubleMatrix(m1,m1,mxREAL); 
      plhs[0] = mxCreateDoubleMatrix(m1,m1,mxREAL);
      plhs[1] = mxCreateDoubleMatrix(m1,1,mxREAL);
      plhs[2] = mxCreateDoubleMatrix(m1-1,1,mxREAL);
      plhs[3] = mxCreateDoubleMatrix(m1,1,mxREAL);
      workarray[0] = mxCreateDoubleMatrix(m1,1,mxREAL);
      workarray[1] = mxCreateDoubleMatrix(m1,1,mxREAL);
      workarray[2] = mxCreateDoubleMatrix(m1,1,mxREAL); 
   }
   else {
      array_ptr =  mxCreateDoubleMatrix(m1,m1,mxCOMPLEX);
      plhs[0] = mxCreateDoubleMatrix(m1,m1,mxCOMPLEX);
      plhs[1] = mxCreateDoubleMatrix(m1,1,mxREAL);
      plhs[2] = mxCreateDoubleMatrix(m1-1,1,mxCOMPLEX);
      plhs[3] = mxCreateDoubleMatrix(m1,1,mxREAL);
      workarray[0] = mxCreateDoubleMatrix(m1,1,mxCOMPLEX);
      workarray[1] = mxCreateDoubleMatrix(m1,1,mxCOMPLEX);
      workarray[2] = mxCreateDoubleMatrix(m1,1,mxCOMPLEX);  
   }

/* Assign pointers to the various parameters */
   
   A = (double *)mxGetPr(array_ptr); 
   memcpy(A,mxGetPr(prhs[0]),m1*m1*sizeof(double)); 
   L    = mxGetPr(plhs[0]);
   alp  = mxGetPr(plhs[1]);
   b = mxGetPr(plhs[2]);
   piv  = mxGetPr(plhs[3]); 
   h = mxGetPr(workarray[0]); 
   v = mxGetPr(workarray[1]); 
   l = mxGetPr(workarray[2]); 
   if (iscmpA) {
      Ai = (double *)mxGetPi(array_ptr); 
      memcpy(Ai,mxGetPi(prhs[0]),m1*m1*sizeof(double)); 
      Li    = mxGetPi(plhs[0]);
      bi = mxGetPi(plhs[2]);
      hi = mxGetPi(workarray[0]); 
      vi = mxGetPi(workarray[1]); 
      li = mxGetPi(workarray[2]); 
   }

/* Do the actual computations in a subroutine */

   if (!iscmpA) {
      assen_real(m1,A,L,alp,b,piv,h,v,l); }
   else {
      assen_cmp(m1,A,L,alp,b,piv,h,v,l,Ai,Li,bi,hi,vi,li); }
   mxDestroyArray(array_ptr);
   return;
}
/**********************************************************/
