%%*****************************************************************
%% plotgap: plot the convergence curve of 
%%          duality gap and infeasibility measure.
%%
%%         h = plotgap(Gap,Feas); 
%%
%% Input:  Gap  = each row of Gap corresponds to a convergence curve
%%                of the duality gap for an SDP.
%%         Feas = each row of Feas corresponds to a convergence curve
%%                of the infeasibility measure for an SDP.
%%
%% Output: h = figure handle. 
%%
%% SDPT3: version 2.1 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 7 Jul 99
%%*****************************************************************

    function  h = plotgap(Gap,Feas); 

    clf; 
    set(0,'defaultaxesfontsize',12);
    set(0,'defaultlinemarkersize',2);
    set(0,'defaulttextfontsize',12);
%%
%% get axis scale for plotting duality gap
%%
    tmp = []; 
    for k = 1:size(Gap,1);
        eval(['g = Gap(k,:);']); 
        if ~isempty(g); 
           idx = find(g > 5*eps); g = g(idx);
           tmp = [tmp abs(g)];
           iter(k) = length(g); 
        else; 
           iter(k) = 0; 
        end;
    end;
    ymax = exp(log(10)*(round(log10(max(tmp)))+0.5));  
    ymin = exp(log(10)*min(floor(log10(tmp)))-0.5);  
    xmax = 5*ceil(max(iter)/5);
%%
%% plot duality gap
%%
    color = '-r --b--m-c '; 
    if nargin == 2; subplot('position',[0.05 0.3 0.45 0.45]); end;
    for k = 1:size(Gap,1);
        eval(['g = Gap(k,:);']); 
        if ~isempty(g); 
           idx = find(g > 5*eps); 
           if ~isempty(idx); g = g(idx); len = length(g); 
              semilogy(len-1,g(len),'.b','markersize',12); hold on;
              h(k) = semilogy(idx-1,g,color([3*(k-1)+1:3*k]),'linewidth',2); 
           end; 
        end;
    end;
    title('duality gap');  axis('square'); 
    if nargin == 1;  axis([0 xmax ymin ymax]); end; 
    hold off;
%%
%% get axis scale for plotting infeasibility
%%
    if nargin == 2; 
       tmp = []; 
       for k = 1:size(Feas,1);  
           eval(['f = Feas(k,:);']); 
           if ~isempty(f); 
              idx = find(f > 0); f = f(idx);
              tmp = [tmp abs(f)];
              iter(k) = length(f);
           else; 
              iter(k) = 0; 
           end;
       end;
       fymax = exp(log(10)*(round(log10(max(tmp)))+0.5));  
       fymin = exp(log(10)*(min(floor(log10(tmp)))-0.5));  
       ymax = max(ymax,fymax); ymin = min(ymin,fymin); 
       xmax = 5*ceil(max(iter)/5);
       axis([0 xmax ymin ymax]);
%%
%% plot infeasibility
%%
       subplot('position',[0.5 0.3 0.45 0.45]);
       for k = 1:size(Feas,1); 
           eval(['f = Feas(k,:);']);
           f(1) = max(f(1),eps); 
           if ~isempty(f); 
              idx = find(f > 1e-20); 
              if ~isempty(idx); f = f(idx); len = length(f); 
                 semilogy(len-1,f(len),'.b','markersize',12); hold on;
                 h(k) = semilogy(idx-1,f,color([3*(k-1)+1:3*k]),'linewidth',2);
              end; 
           end;
       end;
       title('infeasibility measure'); 
       axis('square');  axis([0 xmax ymin max(1,ymax)]);
       hold off; 
    end;  
%%====================================================================


