%%*******************************************************************
%%  Read in a problem in SeDuMi format.
%%
%%  [blk,A,C,b] = read_sedumi(fname)
%%
%%  Input: fname.mat = name of the file containing SDP data in
%%                     SeDuMi format.
%%
%%  WARNING: This routine has not been very thoroughly tested.
%%
%%
%% SDPT3: version 2.1
%% Copyright (c) 2000 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 21 Jan 00
%%******************************************************************

  function [blk,A,C,b]=read_sedumi(fname);
%%
%%  First, load the matlab file containing At, c, b, and K
%%
  K.q = [];
  K.l = [];
  A = 0;
  At = 0;
  fname2 = fname;
  compressed = 0;
  if exist([fname2,'.mat']),
     compressed = 0;
     eval(['load ',fname2,';']);
  elseif exist([fname2,'.mat.gz']);
     compressed = 1;
     unix(['gunzip ',fname2,'.mat.gz']);
     eval(['load ',fname2,';']);
     unix(['gzip ',fname2,'.mat']);
  else
     fprintf('Problem not found. Aborting... \n');
     return;
  end;
  K

  if (size(c,1) == 1), c = c'; end;
  if (size(b,1) == 1), b = b'; end;
  if (At == 0), At = A'; end;  clear A;
  [nn,mm] = size(At); if (max(size(c)) == 1); c = c*ones(nn,1); end;

  if isfield(K,'q'),
     if (K.q > 0), break; end;
  end;

  m = length(b);
  blockstructure = K.s;
  nblocks = length(blockstructure);
%%
%%  Next, set up blk
%%
%%  We also find a map from blockstructure to blk
%%
  dense_blk=blockstructure(find(blockstructure>30))';
  sparse_blk=blockstructure(find((blockstructure<=30) & (blockstructure>0)))';
  diag_blk=[];
  if isfield(K,'l'),
     if (K.l > 0), diag_blk = K.l; end;
  end;
%%
   map=cell(nblocks,1);
   no_of_dense=0;
   no_of_sparse=0;
   no_of_diag=0;
   for i=1:nblocks;
      if blockstructure(i)>30
         map{i}=no_of_dense+1;
         no_of_dense=no_of_dense+1;
      elseif blockstructure(i)<0
         map{i}=[length(dense_blk)+any(sparse_blk)+1,no_of_diag+1,0];
         no_of_diag=no_of_diag+1;
      else
         map{i}=[length(dense_blk)+1,no_of_sparse+1];
         no_of_sparse=no_of_sparse+1;
      end;
   end;
   blk = [];
   if ~isempty(dense_blk);
      for k = 1:length(dense_blk);
          blk{k,1} = 'nondiag';  blk{k,2} = dense_blk(k);
      end;
   end;
   if ~isempty(sparse_blk);
      tmp = size(blk,1);
      blk{tmp+1,1} = 'nondiag';
      blk{tmp+1,2} = sparse_blk;
      cums = [0,cumsum(sparse_blk)];
    end;
   if ~isempty(diag_blk);
      tmp = size(blk,1);
      blk{tmp+1,1} = 'diag';
      blk{tmp+1,2} = sum(diag_blk);
      cumd = [0,cumsum(diag_blk)];
   end;
   N = size(blk,1);
%%
%%  Set up the elements of C and A
%%
  C = cell(N,1);
  A = cell(N,m);
  pos = 0;
  posv = 0;
  for i=1:N,
     if strcmp(blk{i,1},'nondiag')
        ni = sum(blk{i,2});
        C{i}=sparse(ni,ni);
        li = length(blk{i,2});
        for bi = 1:li,
           ki = blk{i,2}(bi);
            for j = 1:ki;
               C{i}(pos+1:pos+ki,pos+j) = c(posv+(j-1)*ki+1:posv+j*ki); 
            end;
        pos = pos + ki;
        posv = posv + ki*ki;
        end;
        C{i} = (C{i} + C{i}')/2;
        for k=1:m
           A{i,k}=sparse(ni,ni);
        end;
     else
        ni=blk{i,2};
        C{i}=c(posv+1:posv+ni);
        for k=1:m,
           A{i,k}=zeros(ni,1);
       end;
     end;
  end;
%%
%%  Next, read in the elements of the various matrices.
%%
  for k = 1:m,
  pos = 0;
  posv = 0;
  ak = At(:,k);
  for i=1:N,
     if strcmp(blk{i,1},'nondiag')
        ni = sum(blk{i,2});
        li = length(blk{i,2}); 
        Ak = sparse(ni,ni);
        for bi = 1:li, 
           ki = blk{i,2}(bi); 
           for j = 1:ki; 
              Ak(pos+1:pos+ki,pos+j) = ak(posv+(j-1)*ki+1:posv+j*ki); 
           end; 
           pos = pos + ki;
           posv = posv + ki*ki; 
           Ak = (Ak + Ak')/2;
           A{i,k}=Ak;
        end;
     else
        ni=blk{i,2};
        A{i,k}=ak(posv+1:posv+ni);
     end;
  end; 
  end;
  return;
%%*******************************************************************
