%%********************************************************* 
%% schurhlf: solve the Schur complement equation
%%
%%  [dy,dtau,Uchol,Lsch,Usch,Psch] = ...
%%  schurhlf(schur,rhs,b,kap,tau,mu,ispd,Uchol,Lsch,Usch,Psch); 
%%    
%% SDPT3: version 2.1 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 7 Jul 99
%%*********************************************************

 function [dy,dtau,Uchol,Lsch,Usch,Psch] = schurhlf(schur,rhs,b,kap,tau,mu,...
                                           ispd,Uchol,Lsch,Usch,Psch); 
   
    m = length(rhs)-1;
    if (mu < 1e-4); 
       if ~isempty(schur); 
          if ispd; 
             [Uchol,indef] = chol(schur(1:m,1:m)); 
             if indef; 
                clear Uchol; 
                Uchol = []; 
                [Lsch,Usch,Psch] = lu(schur(1:m,1:m)); Psch = sparse(Psch);
             else;
                Lsch = []; Usch = []; Psch = []; 
             end;
          else;
             Uchol = [];
             [Lsch,Usch,Psch] = lu(schur(1:m,1:m)); Psch = sparse(Psch); 
          end;
       end; 
       clear schur; 
       if ~isempty(Uchol);
          dy = (rhs(1:m)' / Uchol)'; 
          dy = Uchol \ dy;
       else; 
          dy = Lsch \(Psch * rhs(1:m)); 
          dy = Usch \ dy; 
       end;
       dtau = 0;
    else; 
       if ~isempty(schur);
          if ispd; 
             [Uchol,indef] = chol(schur);    
             if indef; 
                clear Uchol; 
                Uchol = []; 
                [Lsch,Usch,Psch] = lu(schur); Psch = sparse(Psch);
             else; 
                Lsch = []; Usch = []; Psch = [];
             end;
          else;
             Uchol = [];
             [Lsch,Usch,Psch] = lu(schur); Psch = sparse(Psch);
          end;
       end;      
       clear schur;   
       alp = kap/(2*tau);  
       if ~isempty(Uchol);
          dy1 = (rhs' / Uchol)'; 
          dy1 = Uchol \ dy1;
          tmp = ([zeros(m,1); 1]' / Uchol)'; 
          invAU(:,2) = Uchol \ tmp; 
          tmp = ([-b; 0]' / Uchol)'; 
          invAU(:,1) = Uchol \ tmp + alp*invAU(:,2);
       else; 
          dy1 = (Lsch \(Psch * rhs)); 
          dy1 = Usch \ dy1; 
          tmp = (Lsch \ (Psch*[zeros(m,1); 1])); 
          invAU(:,2) = Usch \ tmp; 
          tmp = (Lsch \ (Psch*[-b; 0])); 
          invAU(:,1) = Usch \ tmp + alp*invAU(:,2);
       end;
       %%
       %% Sherman-Morrison formula
       %% B = A + UV', 
       %% invB = invA - (invAU) inv(I + V'invAU) V' invA
       %%
       K = eye(2) + [zeros(1,m) 1; b' alp]*invAU; 
       tmp  = dy1 - invAU*(K \ [zeros(1,m) 1; b' alp]*dy1);
       dy   = tmp(1:m); 
       dtau = tmp(m+1); 
    end; 
%%========================================================== 



