%%*****************************************************************
%% Examples of SDP. 
%%
%% this is an illustration on how to use our SDP solvers
%% coded in sdp.m and sdphlf.m
%% 
%% feas = 1 if want feasible initial iterate
%%      = 0 otherwise
%% algo_opt = 1 if want the usual infeasible path-following methods
%%          = 2 if want the homogeneous and self-dual methods. 
%% 
%% SDPT3: version 2.1
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last Modified: 7 Jul 99
%%*****************************************************************
  
   randn('seed',0); rand('seed',0); 
   feas = input('using feasible starting point? [yes = 1, no = 0] ');
   txt = sprintf('using which methods?\n');
   txt = [txt  sprintf(' 1 if using infeasible path-following algorithm,\n')];
   txt = [txt  sprintf(' 2 if using homogeneous algorithms. ')];
   algo_opt = input(txt);

   if feas == 0; 
      fprintf('\n  using infeasible starting point\n\n');
   else
      fprintf('\n  using feasible starting point\n\n');
   end; 
   pause(1);

   ntrials = 5;
   iterm = zeros(4,8); infom = zeros(4,8); timem = zeros(4,8);

   OPTIONS.gaptol = 1e-13;
   OPTIONS.inftol = 1e-13; 

   for trials = [1:ntrials];
   for eg = [1:8]
       if (eg == 1);
          disp('******** random sdp **********')
          n = 10; m = 10; 
          [blk,A,C,b,X0,y0,Z0] = randsdp(n,[],[],m,feas); 
          text = 'random SDP';
       elseif (eg == 2);
          disp('******** Norm minimization problem. **********')
          n = 10; m = 5; B = []; 
          for k = 1:m+1; B{k} = randn(n); end;
          [blk,A,C,b,X0,y0,Z0] = norm_min(B,feas); 
          text = 'Norm min. pbm';
       elseif (eg == 3); 
          disp('****** Chebyshev approx. problem of a complex matrix ******')
          N = 10; m = 5; 
          B = sprandn(N,N,0.1) + sqrt(-1)*sprandn(N,N,0.1);
          [blk,A,C,b,X0,y0,Z0] = chebymat(B,m,feas); 
          text = 'Chebyshev approx. of a complex matrix';
       elseif (eg == 4); 
          disp('******** Max-cut *********');
          N = 10;
          B = graph(N); 
          [blk,A,C,b,X0,y0,Z0] = maxcut(B,feas); 
          text = 'Maxcut'; 
       elseif (eg == 5); 
          disp('********* ETP ***********')
          N = 10; 
          B = randn(N); B = B*B';
          [blk,A,C,b,X0,y0,Z0] = etp(B,feas); 
          text = 'ETP';
       elseif (eg == 6);
          disp('**** Lovasz theta function ****')  
          N = 10;
          B = graph(N); 
          [blk,A,C,b,X0,y0,Z0] = theta(B,feas);
          text = 'Lovasz theta fn.';
       elseif (eg == 7); 
          disp('**** Logarithmic Chebyshev approx. pbm. ****')  
          N = 20; m = 5; 
          B = rand(N,m); f = rand(N,1); 
          [blk,A,C,b,X0,y0,Z0] = logcheby(B,f,feas);
          text = 'Log. Chebyshev approx. pbm';
       elseif (eg == 8); 
          disp('**** Chebyshev approx. on complex plane ****')  
          N = 20; m = 5; 
          d = randn(N,1) + sqrt(-1)*randn(N,1); 
          [blk,A,C,b,X0,y0,Z0] = chebyinf(d,m,feas);
          text = 'Chebyshev approx. on C';
       end;     
%%
       m = length(b);
       nn = 0;
       for p = 1:size(blk,1),
           nn = nn + sum(blk{p,2});
       end;
       mn = [m,nn]
%%
       Gap = []; Feas = []; legendtext = []; 
       for vers = [1 2 3 4];
           OPTIONS.vers = vers; 
           if (algo_opt == 1); 
              [obj,X,y,Z,gaphist,infeashist,infoall] = ...
              sdp(blk,A,C,b,X0,y0,Z0,OPTIONS);
           elseif (algo_opt == 2);
              [obj,X,y,Z,gaphist,infeashist,infoall] = ...
              sdphlf(blk,A,C,b,X0,y0,Z0,1,1,OPTIONS);
           end;
           eval(['Gap(',num2str(vers),',1:length(gaphist)) = gaphist;']); 
           eval(['Feas(',num2str(vers),',1:length(infeashist))=infeashist;']);
           if (vers==1);     legendtext = [legendtext, ' ,''AHO'' ']; 
           elseif (vers==2); legendtext = [legendtext, ' ,''HKM'' '];
           elseif (vers==3); legendtext = [legendtext, ' ,''NT''  ']; 
           elseif (vers==4); legendtext = [legendtext, ' ,''GT''  '];
           end;                
           infom(vers,eg)=infom(vers,eg) + infoall(1);
           iterm(vers,eg)=iterm(vers,eg) + infoall(2);
           timem(vers,eg)=timem(vers,eg) + infoall(5);
       end;
       h = plotgap(Gap,Feas);      
       xlabel(text);        
       eval(['legend(h(h>0)' ,legendtext, ')']);
       disp('**** press enter to continue ****'); pause
   end; 
   end; 
%%
   infom = infom/ntrials;
   iterm = iterm/ntrials;
   timem = timem/ntrials;
   infor = infom
   iterr = iterm
   timer = timem
%%======================================================================
