%%***********************************************************************
%% validate: validate data
%%
%%  [A,C,X0,Z0,isspA,cmp,nnzA] = validate(blk,A,C,b,X0,y0,Z0,spdensity);
%% 
%% SDPT3: version 2.1 y
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 7 Jul 99 
%%***********************************************************************

function  [A,C,X0,Z0,isspA,cmp,nnzA] = validate(blk,A,C,b,X0,y0,Z0,spdensity);

   if (nargin == 7); spdensity = 0.3; end;

   if ~isreal([b; y0]); error('both b and y0 must be real'); end; 
   if (~iscell(blk) | size(blk,2)~=2); 
      error('blk: must be a cell array with 2 columns'); end; 
   if ~iscell(A); 
      error('A must be a cell array with A{k} = A_k of the SDP problem'); 
   end; 
   if iscell(C) & size(C,2)~=1; 
      error('cell array C can only have 1 column'); end;
   if iscell(X0) & size(X0,2)~=1; 
      error('cell array X0 can only have 1 column'); end;
   if iscell(Z0) & size(Z0,2)~=1; 
      error('cell array Z0 can only have 1 column'); end;
%%
%%
   if ~(iscell(C) | iscell(X0) | iscell(Z0)); 
      m  = length(b);  
      if strcmp(blk{1},'nondiag');
         [n,q] = size(C); 
         if (sum(blk{2}) ~= n); 
            error('block size not correctly specified'); end; 
         if (n~= q); error('C must be a square matrix'); end; 
         if (length(y0) ~= m); error('length of y0 is not correct'); end; 
         if (norm([C-C',X0-X0',Z0-Z0'],inf) > 2e-13); 
            error(' X0, Z0, C not all symmetric'); 
         end;
         isrealA = 1;  
         for k = 1:size(A,2)
            Ak = A{k}; 
            if any(size(Ak)-[n n]); 
               error(['size of A{k} not equal to',num2str(n)]);
            end;
            if (norm(Ak-Ak',inf) > 2e-13);
                error(['A{k} is not symmtric']);
            end; 
            nnzA(1,k) = nnz(Ak); 
            if (length(blk{2}) == 1); 
               if (nnzA(k) > spdensity*n*n) 
  	          if issparse(Ak); A{k} = full(Ak); end;
               else; 
                  if ~issparse(Ak); A{k} = sparse(Ak); end;
               end; 
	    elseif (length(blk{2}) > 1) 
               if ~issparse(Ak); A{k} = sparse(Ak); end;
            end;  
            isrealA = min(isrealA, isreal(Ak));
         end; 
         isspA = (sum(nnzA) <= spdensity*m*n*n); 
         cmp = 1-min(isrealA,isreal(C)); 
         if (nnz(C) > spdensity*n*n); 
            if issparse(C); C = full(C); end;
         else
            if ~issparse(C); C = sparse(C); end; 
         end;
         if (nnz(X0) > spdensity*n*n); 
            if issparse(X0); X0 = full(X0); end;
         else
            if ~issparse(X0); X0 = sparse(X0); end; 
         end;
         if (nnz(Z0) > spdensity*n*n); 
            if issparse(Z0); Z0 = full(Z0); end;
         else
            if ~issparse(Z0); Z0 = sparse(Z0); end; 
         end;
      elseif strcmp(blk{1},'diag'); 
         if ~all([size(C,2) size(X0,2) size(Z0,2)]==1); 
            error('blk = ''diag'', C,X0,Z0 must be column vectors'); 
         end;
         n = length(C);
         for k = 1:size(A,2)
            Ak = A{k}; 
            if (size(Ak,2)~=1); 
               error('blk = ''diag'', A{i,j} must be column vector'); 
            end;
            nnzA(1,k) = nnz(Ak); 
            if (nnzA(k) > spdensity*n);
	       if  issparse(Ak); A{k} = full(Ak); end;
            else;
               if ~issparse(Ak); A{k} = sparse(Ak); end; 
            end;
         end;
         isspA = (sum(nnzA) < spdensity*m*n); 
         cmp = 0; 
         if (nnz(C) > spdensity*n); 
            if issparse(C); C = full(C); end;
         else
            if ~issparse(C); C = sparse(C); end; 
         end;
         if (nnz(X0) > spdensity*n); 
            if issparse(X0); X0 = full(X0); end;
         else
            if ~issparse(X0); X0 = sparse(X0); end; 
         end;
         if (nnz(Z0) > spdensity*n); 
            if issparse(Z0); Z0 = full(Z0); end;
         else
            if ~issparse(Z0); Z0 = sparse(Z0); end; 
         end;
      else 
         error(' blk: some fields not specified correctly'); 
      end;
   else; 
      for p = 1:size(blk,1); 
         [A(p,:),C{p},X0{p},Z0{p},isspA(p),cmp(p),nnzA(p,:)] = ...
         validate(blk(p,:),A(p,:),C{p},b,X0{p},y0,Z0{p});
      end;
   end;
%%=============================================================



