%%*******************************************************************
%% eqGHKMsch: compute schur matrix of HKM direction
%%            for eqG problems.  
%%
%%  [schur,hRd] = eqGHKMsch(X,Zinv,Rd,schurfun_parms); 
%%
%%  Ak = -ek*ek'                 if k = 1:m-1
%%     = em*em' -[e; 0]*[e; 0]'  if k = m. 
%%
%% SDPT3: version 2.1
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last Modified: 7 Jul 99
%%*******************************************************************
 
    function [schur,hRd] = eqGHKMsch(X,Zinv,Rd,schurfun_parms); 

    if ~exist('Rd'); Rd = []; hRd = []; end;

    x = X{1};        clear X;
    zinv = Zinv{1};  clear Zinv; 
%%
%% schur = A*EinvF(AT) 
%%
    m = size(x,1);
    e = [ones(m-1,1); 0];
    xe = x*e; 
    xem = x(:,m);
    ze = zinv*e;  
    zem = zinv(:,m);
    schur = x .* zinv;
    schur(:,m) = xe.*ze - xem.*zem;
    schur(m,:) = schur(:,m)';
    schur(m,m) = (e'*xe)*(e'*ze) -2*(e'*xem)*(e'*zem) +xem(m)*zem(m);
%%
%%  A*EinvF(Rd)
%%
    if ~isempty(Rd); 
       hRd = zeros(m,1);
       tmp = Rd{1} * x; 
       clear Rd; 
       hRd(m) = -ze'*(tmp*e) + zem'*tmp(:,m);
       for k=1:m-1; hRd(k) = -zinv(k,:)*tmp(:,k); end;
    end;
%%*******************************************************************
