%%****************************************************
%% eqGNTsch: compute schur matrix of NT direction
%%           for eqG problems.  
%%
%%  [schur,hRd] = eqGNTsch(W,Rd,schurfun_parms); 
%%
%%  Ak = -ek*ek'                 if k = 1:m-1
%%     = em*em' -[e; 0]*[e; 0]'  if k = m. 
%%
%% SDPT3: version 2.1
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last Modified: 7 Jul 99
%%****************************************************
 
    function [schur,hRd] = eqGNTsch(W,Rd,schurfun_parms); 

    if ~exist('Rd'); Rd = []; hRd = []; end; 

    w = W{1}; clear W; 

    m = size(w,1);
    e = [ones(m-1,1); 0];
    we = w*e;
    wem = w(:,m);
    schur = w .* w;
    schur(:,m) = we.*we - wem.*wem;
    schur(m,:) = schur(:,m)';
    schur(m,m) = (e'*we)*(e'*we) - 2*(e'*wem)*(e'*wem) ...
                 + wem(m)*wem(m);
    if ~isempty(Rd); 
       hRd = zeros(m,1);
       tmp = Rd{1} * w; 
       hRd(m) = -we'*(tmp*e) + wem'*tmp(:,m);
       for k=1:m-1; hRd(k) = -w(k,:)*tmp(:,k); end;       
    end;
%%****************************************************
