%%****************************************************
%% gppGTsch: compute schur matrix of GT direction
%%           for GPP problems.  
%%
%% [schur,hRd] = gppGTsch(invZHt,Dsch,Rd,schurfun_parms); 
%%
%%  Ak = -e*e'   if k = 1
%%     = -ek*ek' if k > 1. 
%%****************************************************

  function [schur,hRd] = gppGTsch(invZHt,Dsch,Rd,schurfun_parms); 
  
   if ~exist('Rd'); Rd = []; hRd = []; end; 

   zz = invZHt{1};  clear invZHt;   
   E = Dsch{1};     clear Dsch; 

   n = size(zz,1); 
   e = ones(n,1); 
   m = n+1; 
   schur = zeros(m);
   zze = zz*e; 
   for i = 2:m 
      zi = zz(:,i-1);
      for j = 2:i
         w = zi.*zz(:,j-1); 
         schur(i,j) = w'*E*w; 
      end;
      w = zze .* zi; 
      schur(i,1) = w'*E*w; 
   end;       
   w = zze .* zze; 
   schur(1,1) = w'*E*w; 
   schur = schur + tril(schur,-1)'; 

   if ~isempty(Rd);
     hRd = zeros(m,1); 
     blk{1} = 'nondiag';  blk{2} = n; 
     tmp = Prod3(blk, zz, Rd{1}, zz' , 1); 
     tmp = E.*tmp; 
     clear Rd;
     hRd(1) = -zze'*(tmp*zze); 
     for k=1:n; 
         hRd(k+1) = -zz(:,k)'*(tmp*zz(:,k)); 
     end;
  end;   
%%****************************************************
