%%****************************************************
%% gppHKMsch: compute schur matrix of HKM direction
%%            for GPP problems.  
%%
%% [schur,hRd] = gppHKMsch(X,Zinv,Rd,schurfun_parms); 
%%
%%  Ak = -e*e'   if k = 1
%%     = -ek*ek' if k > 1. 
%%
%% SDPT3: version 2.1
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last Modified: 7 Jul 99
%%****************************************************

  function [schur,hRd] = gppHKMsch(X,Zinv,Rd,schurfun_parms); 
  
   if ~exist('Rd'); Rd = []; hRd = []; end; 
   x = X{1};       clear X; 
   zinv = Zinv{1}; clear Zinv; 
   
   n = size(x,1); 
   e = ones(n,1); 
   m = n+1; 
   schur = zeros(m); 
   schur(2:m,2:m) = x .* zinv;  
   tmp = (x*e) .* (zinv*e); 
   schur(2:m,1) = tmp; 
   schur(1,2:m) = tmp'; 
   schur(1,1)   = (e'*x*e) * (e'*zinv*e); 
   
   if ~isempty(Rd);
     hRd = zeros(m,1); 
     tmp = Rd{1} * x; 
     clear Rd;
     hRd(1) = -(e'zinv)*(tmp*e); 
     for k=1:n; hRd(k+1) = -zinv(k,:)*tmp(:,k); end; 
  end;   
%%****************************************************
