%%****************************************************
%% gppNTsch: compute schur matrix of NT direction
%%           for GPP problems.  
%%
%%  [schur,hRd] = gppHKMsch(X,Zinv,Rd,schurfun_parms); 
%%
%%  Ak = -e*e'   if k = 1
%%     = -ek*ek' if k > 1. 
%%
%% SDPT3: version 2.1
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last Modified: 7 Jul 99
%%****************************************************

  function [schur,hRd] = gppNTsch(W,Rd,schurfun_parms); 

   if ~exist('Rd'); Rd = []; hRd = []; end;   
   w = W{1};  clear W; 
   n = size(w,1); 
   m = n+1; 
   e = ones(n,1); 
   schur = zeros(m); 
   schur(2:m,2:m) = w .* w; 
   we = w*e; 
   tmp = we .* we; 
   schur(2:m,1) = tmp;
   schur(1,2:m) = tmp'; 
   schur(1,1)   = (e'*we)^2; 

   if ~isempty(Rd); 
      hRd = zeros(m,1); 
      tmp = Rd{1} * w; 
      clear Rd; 
      hRd(1) = -(e'*w)*(tmp*e); 
      for k=1:n; hRd(k+1) = -w(k,:)*tmp(:,k); end; 
   end;
%%****************************************************
