%%******************************************************************
%% thetaGTsch: compute schur matrix of GT direction
%%              for Lovasz theta problems.  
%%
%%  [schur,hRd] = thetaGTsch(invZHt,Dsch,Rd,schurfun_parms); 
%%
%%  Ak = -I   if k = 1
%%     = -1/2(ei*ej'+ej*ei') if k > 1 and Ak corresponds to arc i-j.
%%
%% SDPT3: version 2.1
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last Modified: 7 Jul 99 
%%******************************************************************

  function [schur,hRd] = thetaGTsch(invZHt,Dsch,Rd,schurfun_parms); 
  
   if isempty(schurfun_parms); 
      error('thetaGTsch: need parameters in OPTIONS.schurfun_parms.'); 
   end;
   listA = schurfun_parms; %% a cell array such that listA{k} contains
                           %% the indices the nonzeros elements of A{k}. 
%%
   if ~exist('Rd'); Rd = []; hRd = []; end; 
   zz = invZHt{1};   clear invZHt; 
   E  = Dsch{1};     clear Dsch; 
   m = length(listA); 

   idx1 = zeros(m-1,2); 
   idx2 = zeros(m-1,2); 
   for i=2:m
      idx1(i-1)=listA{i}(1,1);
      idx2(i-1)=listA{i}(1,2); 
   end; 
    
   schur=zeros(m);
   blk{1} = 'nondiag'; blk{2} = size(zz,1); 
   for i = 2:m
      zz1 = zz(:,idx1(i-1));
      zz2 = zz(:,idx2(i-1));
      tmp = zz1*zz2' + zz2*zz1';
      tmp = Prod3(blk,zz',tmp.*E,zz,1);  
      for j = 2:i
         schur(i,j) = tmp(idx1(j-1),idx2(j-1));
      end;
   end;
   schur = 0.5*schur; 
%%
   W = zz*zz'; 
   for k=2:m
      i = listA{k}(1,1); 
      j = listA{k}(1,2);
      zz1 = zz(:,i); 
      zz2 = zz(:,j); 
      tmp = zz1*zz2' + zz2*zz1'; 
      schur(k,1) = 0.5*blktrace(blk,tmp.*E, W);
   end;
   schur(1,1) = blktrace(blk, W.*E, W); 
   schur = schur + tril(schur,-1)'; 
%% 
   if ~isempty(Rd);
      hRd = zeros(m,1); 
      Rdzz = Prod3(blk,zz,Rd{1},zz',1); 
      clear Rd; 
      for k=2:m
         i = listA{k}(1,1); 
         j = listA{k}(1,2);
         zz1 = zz(:,i); 
         zz2 = zz(:,j); 
         tmp = zz1*zz2' + zz2*zz1';
         hRd(k) = -0.5*blktrace(blk, tmp.*E, Rdzz);;      
      end; 
      hRd(1) = -blktrace(blk, W.*E, Rdzz); 
   end;   
%%
%%******************************************************************
