%%******************************************************************
%% thetaHKMsch: compute schur matrix of HKM direction
%%              for Lovasz theta problems.  
%%
%% [schur,hRd] = thetaHKMsch(X,Zinv,Rd,schurfun_parms); 
%%
%%  Ak = -I   if k = 1
%%     = -1/2(ei*ej'+ej*ei') if k > 1 and Ak corresponds to arc i-j. 
%%
%%  the parameter variable listA can be generated as follows:
%%
%%  >> [listA,splistA,permA,listRd] = nonzerolist(blk,A,1);
%%  >> listA(permA) = listA; 
%%
%% SDPT3: version 2.1
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last Modified: 7 Jul 99
%%******************************************************************

  function [schur,hRd] = thetaHKMsch(X,Zinv,Rd,schurfun_parms);
  
   if isempty(schurfun_parms); 
      error('thetaHKMsch: need parameters in OPTIONS.schurfun_parms.'); 
   end;
   listA = schurfun_parms; %% a cell array such that listA{k} contains
                           %% the indices the nonzeros elements of A{k}. 
%%
   if ~exist('Rd'); Rd = []; hRd = []; end; 
   x = X{1};        clear X; 
   zinv = Zinv{1};  clear Zinv;

   m = length(listA); 
   idx1 = zeros(m-1,1); 
   idx2 = zeros(m-1,1); 
   for i=2:m
      idx1(i-1) = listA{i}(1,1);
      idx2(i-1) = listA{i}(1,2);  
   end;
    
   schur=zeros(m);
   tmp = x(idx1,idx1).*zinv(idx2,idx2) + x(idx2,idx1).*zinv(idx1,idx2) ...
         + x(idx1,idx2).*zinv(idx2,idx1) + x(idx2,idx2).*zinv(idx1,idx1);
   tmp = 0.25*tmp;
   schur(2:m,2:m) = tmp;
   clear tmp;
%%
   for k=2:m
      i1 = idx1(k-1); 
      j1 = idx2(k-1); 
      schur(1,k) = 0.5*(x(i1,:)*zinv(:,j1)+x(j1,:)*zinv(:,i1));
      schur(k,1) = schur(1,k);
   end;
   blk{1,1} = 'nondiag'; blk{1,2} = size(x,1);
   schur(1,1) = blktrace(blk,zinv,x);
%%  
  if ~isempty(Rd);
     hRd = zeros(m,1); 
     tmp = Rd{1} * x; 
     clear Rd; 
     for k = 2:m
        i1 = idx1(k-1); 
        j1 = idx2(k-1); 
        hRd(k) = -0.5*(zinv(i1,:)*tmp(:,j1)+zinv(j1,:)*tmp(:,i1));      
     end; 
     hRd(1) = -blktrace(blk,zinv,tmp,0);
  end;   
%%******************************************************************
