%%******************************************************************
%% thetaNTsch: compute schur matrix of NT direction
%%              for Lovasz theta problems.  
%%
%%  [schur,hRd] = thetaNTsch(W,Rd,schurfun_parms); 
%%
%%  Ak = -I   if k = 1
%%     = -1/2(ei*ej'+ej*ei') if k > 1 and Ak corresponds to arc i-j. 
%%
%%  the parameter variable listA can be generated as follows:
%%
%%  >> [listA,splistA,permA,listRd] = nonzerolist(blk,A,1);
%%  >> listA(permA) = listA; 
%%
%% SDPT3: version 2.1
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last Modified: 7 Jul 99
%%******************************************************************

  function [schur,hRd] = thetaNTsch(W,Rd,schurfun_parms); 
  
   if isempty(schurfun_parms); 
      error('thetaNTsch: need parameters in OPTIONS.schurfun_parms.'); 
   end;
   listA = schurfun_parms; %% a cell array such that listA{k} contains
                           %% the indices the nonzeros elements of A{k}. 
%%
    if ~exist('Rd'); Rd = []; hRd = []; end; 
    w = W{1};   clear W; 
    m = length(listA); 

    idx1 = zeros(m-1,1); 
    idx2 = zeros(m-1,1); 
    for i=2:m
       idx1(i-1)=listA{i}(1,1);
       idx2(i-1)=listA{i}(1,2); 
    end;
    
   schur=zeros(m);
   tmp = w(idx1,idx1).*w(idx2,idx2) + w(idx2,idx1).*w(idx1,idx2);
   tmp = 0.5*tmp;
   
   schur(2:m,2:m) = tmp;
   clear tmp;
%%
   for k=2:m
      i1 = idx1(k-1); 
      j1 = idx2(k-1); 
      schur(1,k) = w(i1,:)*w(:,j1);
      schur(k,1) = schur(1,k);
   end;
   blk{1,1} = 'nondiag'; blk{1,2} = size(w,1);
   schur(1,1) = blktrace(blk,w,w);
%% 
   if ~isempty(Rd);
      hRd = zeros(m,1); 
      tmp = Rd{1} * w; 
      clear Rd; 
      for k=2:m
         i1 = idx1(k-1); 
         j1 = idx2(k-1); 
         hRd(k) = -0.5*(w(i1,:)*tmp(:,j1)+w(j1,:)*tmp(:,i1));      
      end; 
      hRd(1) = -blktrace(blk,w,tmp,0);
   end;   
%%
%%******************************************************************
